# -*- coding: utf-8 -*-
"""
Created on Thu Sep  5 15:18:21 2024

@author: glhote1
"""

#%% Imports

import os
from os.path import abspath
from inspect import getsourcefile
import tifffile as tf
import pyqtgraph as pg
from PyQt5.QtWidgets import QApplication
from inichord import general_functions as gf # USED
import numpy as np
from pyqtgraph.Qt import QtGui

from inichord import general_functions as gf

path2thisFile = abspath(getsourcefile(lambda:0))
uiclass, baseclass = pg.Qt.loadUiType(os.path.dirname(path2thisFile) + "/Denoise_2Dmap_ui_01.ui")

class MainWindow(uiclass, baseclass):
    def __init__(self,parent):
        super().__init__()

        self.setupUi(self)
        self.parent = parent
        
        self.OpenData.clicked.connect(self.loaddata) # Load data
        # self.preview.clicked.connect(self.run_Denoising) # Run batch
        self.Push_validate.clicked.connect(self.extract_data)
        
        self.setWindowIcon(QtGui.QIcon('icons/filter_icon.png')) 
        self.defaultIV() # Default ImageView (when no image)
        
        try: # if data is imported from the main GUI
            self.expStack = parent.KAD
            self.denoised_map = np.copy(self.expStack)
            
            self.expStack = self.check_type(self.expStack) # Convert data to float32 if needed
            self.denoised_map = self.check_type(self.denoised_map) # Convert data to float32 if needed   
            
            self.displayExpStack(self.expStack)
            
            self.run_Denoising()
            
        except:
            pass
        
        self.patch_size = 5
        self.patch_distance = 6
        self.param_h = self.slider_h.value() / 10_0.0
        
        self.label_distance.setText("Patch Distance: " + str(self.patch_distance))
        self.label_size.setText("Patch Size: " + str(self.patch_size))
        self.label_h.setText("Parameter h: " + str(self.param_h))
        
        self.slider_size.valueChanged.connect(self.size_changed)
        self.slider_distance.valueChanged.connect(self.distance_changed)
        self.slider_h.valueChanged.connect(self.h_changed)
        
        app = QApplication.instance()
        screen = app.screenAt(self.pos())
        geometry = screen.availableGeometry()
        
        # Position (self.move) and size (self.resize) of the main GUI on the screen
        self.move(int(geometry.width() * 0.1), int(geometry.height() * 0.1))
        self.resize(int(geometry.width() * 0.8), int(geometry.height() * 0.6))
        self.screen = screen

    def loaddata(self):
        StackLoc, StackDir = gf.getFilePathDialog("2D map") # Image importation
        
        self.expStack = [] # Initialization of the variable self.image (which will be the full series folder)
        self.expStack = tf.TiffFile(StackLoc[0]).asarray() # Import the unit 2D array

        self.expStack = np.flip(self.expStack, 0)
        self.expStack = np.rot90(self.expStack, k=1, axes=(1, 0))

        self.denoised_map = np.copy(self.expStack)
        
        self.expStack = self.check_type(self.expStack) # Convert data to float32 if needed
        self.denoised_map = self.check_type(self.denoised_map) # Convert data to float32 if needed   

        self.displayExpStack(self.expStack)
        
        self.run_Denoising()

    def check_type(self,data): # Check if the data has type uint8 or uint16 and modify it to float32
        datatype = data.dtype
        print(datatype)

        if datatype == "float64":
            data = gf.convertToUint8(data)
            self.data = data.astype(np.float32)
        if datatype == "uint16":
            data = gf.convertToUint8(data)
            self.data = data.astype(np.float32)
        elif datatype == "uint8":
            self.data = data.astype(np.float32)
        elif datatype == "float32":
            data = gf.convertToUint8(data)
            self.data = data.astype(np.float32)
            
        return self.data

    def run_Denoising(self):
        a = gf.NonLocalMeanDenoising(self.expStack[:, :], self.param_h, True, self.patch_size, self.patch_distance)
        
        self.denoised_map[:, :] = a

        self.displayExpStack(self.denoised_map)
    
    def size_changed(self):
        value = self.slider_size.value()
        self.patch_size = value
        self.label_size.setText("Patch Size: " + str(value))
        self.run_Denoising()
    
    def distance_changed(self):
        value = self.slider_distance.value()
        self.threshold = value
        self.label_distance.setText("Patch Distance: " + str(value))
        self.run_Denoising()
        
    def h_changed(self):
        value = self.slider_h.value()
        self.param_h = value / 10_0.0        
        self.label_h.setText("Parameter h: " + str(self.param_h))
        self.run_Denoising()  
        
    def extract_data(self):
        self.parent.KAD = np.copy(self.denoised_map) # Copy in the main GUI
        self.parent.StackList.append(self.denoised_map) # Add the data in the stack list
        
        Combo_text = '\u2022 Denoised 2D map'
        Combo_data = self.denoised_map
        self.parent.choiceBox.addItem(Combo_text, Combo_data) # Add the data in the QComboBox

        self.parent.displayDataview(self.parent.KAD) # Display the labeled grain
        self.parent.choiceBox.setCurrentIndex(self.parent.choiceBox.count() - 1) # Show the last data in the choiceBox QComboBox

        self.parent.Info_box.ensureCursorVisible()
        self.parent.Info_box.insertPlainText("\n \u2022 Denoised 2D map.") 
        
        self.parent.Save_button.setEnabled(True)
        self.parent.Reload_button.setEnabled(True)
        self.parent.choiceBox.setEnabled(True)
        
        self.close()

    def defaultIV(self):
        self.expSeries.clear()
        self.expSeries.ui.histogram.hide()
        self.expSeries.ui.roiBtn.hide()
        self.expSeries.ui.menuBtn.hide()
        
        view = self.expSeries.getView()
        view.setBackgroundColor(self.parent.color1)
        
        ROIplot = self.expSeries.getRoiPlot()
        ROIplot.setBackground(self.parent.color1)

    def displayExpStack(self, Series):
        self.expSeries.ui.histogram.hide()
        self.expSeries.ui.roiBtn.hide()
        self.expSeries.ui.menuBtn.hide()
        
        view = self.expSeries.getView()
        state = view.getState()        
        self.expSeries.setImage(Series) 
        view.setState(state)
        
        view.setBackgroundColor(self.parent.color1)
        ROIplot = self.expSeries.getRoiPlot()
        ROIplot.setBackground(self.parent.color1)
        
        font=QtGui.QFont('Noto Sans Cond', 9)
        ROIplot.getAxis("bottom").setTextPen('k') # Apply size of the ticks label
        ROIplot.getAxis("bottom").setTickFont(font)
        
        self.expSeries.timeLine.setPen(color=self.parent.color3, width=15)
        self.expSeries.frameTicks.setPen(color=self.parent.color1, width=5)
        self.expSeries.frameTicks.setYRange((0, 1))

        s = self.expSeries.ui.splitter
        s.handle(1).setEnabled(True)
        s.setStyleSheet("background: 5px white;")
        s.setHandleWidth(5) 


# -*- coding: utf-8 -*-
"""
Created on Wed Nov 22 23:26:20 2023

@author: clanglois1
"""
import os
from inspect import getsourcefile
from os.path import abspath
import numpy as np
import pyqtgraph as pg
from PyQt5.QtWidgets import QApplication
from PyQt5 import QtCore, QtGui
from scipy import ndimage

from inichord import general_functions as gf
from scipy.fft import fft2, ifft2, fftshift

path2thisFile = abspath(getsourcefile(lambda:0))
uiclass, baseclass = pg.Qt.loadUiType(os.path.dirname(path2thisFile) + "/stackModifier_v9_TSG.ui")

class MainWindow(uiclass, baseclass):
    def __init__(self, parent):
        super().__init__()
        self.setupUi(self)
        self.parent = parent # Link sub-gui with main gui
        self.expStack = parent.Current_stack
        
        self.nSlices = self.expStack.shape[0]
        self.indice = self.expStack.shape[0]
        self.stackInfoValues_label.setText(str(self.expStack.shape)) # Shape of the image serie
        
        self.roi = pg.ROI((0, 0), (int(self.expStack.shape[1] * 1), int(self.expStack.shape[2] * 1)))
        self.roi.addScaleHandle((1, 1), (0, 0)) 
        self.roi.addScaleHandle((0, 1), (1, 0)) 
        self.roi.addScaleHandle((1, 0), (0, 1)) 
        self.roi.addScaleHandle((0, 0), (1, 1))
        self.roi.sigRegionChanged.connect(self.updateROIcoord)
        
        self.ROIinfoValues_label.setText(f"W: {int(self.roi.size()[0])} ; H: {int(self.roi.size()[1])} ; Origin: {int(self.roi.pos()[0]), int(self.roi.pos()[1])}")

        self.ROIwidth = int(self.roi.size()[0])
        self.ROIheight = int(self.roi.size()[1])
        
        self.ROIoriX = int(self.roi.pos()[0])
        self.ROIoriY = int(self.roi.pos()[1])
        
        self.expSeries.roiClicked()
        self.expSeries.addItem(self.roi)
        
        self.roi.setPen(color=self.parent.color5)
        
        self.initiateSliceKeeper()
        
        self.x = 0
        self.y = 0
           
        self.crosshair_v1= pg.InfiniteLine(angle=90, movable=False, pen=self.parent.color5)
        self.crosshair_h1 = pg.InfiniteLine(angle=0, movable=False, pen=self.parent.color5)
        
        self.proxy1 = pg.SignalProxy(self.expSeries.scene.sigMouseMoved, rateLimit=60, slot=self.mouseMoved)
        self.proxy4 = pg.SignalProxy(self.expSeries.ui.graphicsView.scene().sigMouseClicked, rateLimit=60, slot=self.mouseClick)
    
        self.start_spinBox.valueChanged.connect(self.modifySliceKeeper)
        self.step_spinBox.valueChanged.connect(self.modifySliceKeeper)
        self.validate_bttn.clicked.connect(self.validate_modified_Stack)
        self.Del_bttn.clicked.connect(self.delete_slice)
        self.Replace_bttn.clicked.connect(self.replace_slice)
        self.Erroneous_bttn.clicked.connect(self.image_correction)
        
        self.setWindowIcon(QtGui.QIcon('icons/crop_icon.png'))
        
        self.displayExpStack(self.expStack)
        
        app = QApplication.instance()
        screen = app.screenAt(self.pos())
        geometry = screen.availableGeometry()
        
        # Position (self.move) and size (self.resize) of the main GUI on the screen
        self.move(int(geometry.width() * 0.1), int(geometry.height() * 0.15))
        self.resize(int(geometry.width() * 0.7), int(geometry.height() * 0.6))
        self.screen = screen

    def initiateSliceKeeper(self):
        self.start_spinBox.setRange(0, self.nSlices)
        self.step_spinBox.setRange(1, self.nSlices)
        self.nbSlice_spinBox.setRange(1, self.nSlices)
        self.nbSlice_spinBox.setValue(self.nSlices)
    
    def modifySliceKeeper(self):
        sender = self.sender()
        
        if sender == self.start_spinBox or sender == self.step_spinBox:
            
            self.indice = 0
            n = 0
            
            while self.indice < (self.nSlices - self.step_spinBox.value() - self.start_spinBox.value()):
                self.indice += self.step_spinBox.value()
                n +=1
                self.nbSlice_spinBox.setRange(1, n + 1)
            self.nbSlice_spinBox.setValue(n + 1)
            
        if sender == self.start_spinBox:
            self.expSeries.setCurrentIndex(self.start_spinBox.value())                

    def updateROIcoord(self):
        self.ROIwidth = int(self.roi.size()[0])
        self.ROIheight = int(self.roi.size()[1])
        
        self.ROIoriX = int(self.roi.pos()[0])
        self.ROIoriY = int(self.roi.pos()[1])
        self.ROIinfoValues_label.setText(f"W: {int(self.roi.size()[0])} ; H: {int(self.roi.size()[1])} ; Origin: {int(self.roi.pos()[0]), int(self.roi.pos()[1])}")

    def delete_slice(self):
        indexes = [self.expSeries.currentIndex] # Index of the slice to delete (current view)
        self.expStack = np.delete(self.expStack, indexes, axis=0) # Remove the slice 
        
        self.stackInfoValues_label.setText(str(self.expStack.shape)) # Extract new stack shape
        self.nSlices = self.expStack.shape[0] # Extract the new number of slices
        self.nbSlice_spinBox.setRange(1, self.nSlices) # Modificiation of the dispatcher
        
        self.displayExpStack(self.expStack)
        
    def replace_slice(self): 
        indexes = self.expSeries.currentIndex # Index of the slice to delete (current view)

        if indexes > 0: # Remplacer le tableau 2D à l'indice 'i' par le tableau 2D de l'indice 'i-1'
            self.expStack[indexes] = self.expStack[indexes - 1]
        elif indexes == 0:
            [indexes] = self.expStack[indexes + 1]
            
        self.displayExpStack(self.expStack)
        
    def image_correction(self):
        self.Threshold_value = self.Threshold_2.value()

        distance = []
    
        # Comparer chaque image avec l'image de référence
        for j in range(0, self.expStack.shape[0]):
            if j == 0:
                image_reference = self.expStack[j]
            else:
                image_reference = self.expStack[j-1]
                
            decalage = self.Compute_shift(image_reference, self.expStack[j])
            distance_decalage = np.linalg.norm(decalage)
            
            distance.append(distance_decalage)
    
        index = np.hstack(distance)
        index = index < self.Threshold_value
        
        # Parcourir l'array pour trouver les paires de 'False'
        for k in range(len(index) - 1):
            # Vérifier si deux éléments consécutifs sont 'False'
            if index[k] == False and index[k + 1] == False:       
                self.expStack[k] = self.expStack[k - 1]
    
        self.displayExpStack(self.expStack)
        
    def Compute_shift(self,image1, image2):
        # Calculer la transformation de Fourier des deux images
        image1 = gf.convertToUint8(image1)
        image2 = gf.convertToUint8(image2)
        
        f_image1 = fft2(image1)
        f_image2 = fft2(image2)

        # Calculer la corrélation croisée en utilisant le produit conjugué
        produit_conjugue = f_image1 * np.conj(f_image2)
        cross_correlation = fftshift(ifft2(produit_conjugue))

        # Trouver le pic de la corrélation croisée pour déterminer le décalage
        decalage = np.unravel_index(np.argmax(np.abs(cross_correlation)), cross_correlation.shape)

        # Calculer le décalage relatif à partir du centre
        centre = np.array(image1.shape) // 2
        decalage = np.array(decalage) - centre

        return decalage

    def validate_modified_Stack(self):
        indStartX = self.ROIoriX
        indStartY = self.ROIoriY
        
        indEndX = indStartX + self.ROIwidth
        indEndY = indStartY + self.ROIheight
        
        toBin_Stack = self.expStack[:, indStartX:indEndX, indStartY:indEndY]
        
        start = self.start_spinBox.value()
        step = self.step_spinBox.value()
        indEnd = start + self.nbSlice_spinBox.value() * step
 
        toBin_Stack = toBin_Stack[start:indEnd:step,:,:]      
        
        reduction = 1 / self.XYbin_spinBox.value()
        splineOrder = self.spline_spinBox.value()
        
        self.binStack = ndimage.zoom(toBin_Stack, (1, reduction, reduction), order = splineOrder)
        
        if len(self.binStack) == 1:
            self.parent.popup_message("Edit tools","Only one slice. Please change the number of images",'icons/crop_icon.png')
        else:
            self.parent.Current_stack = self.binStack
            self.parent.StackList.append(self.binStack)

            Combo_text = '\u2022 Edited stack'
    
            Combo_data = self.binStack
            self.parent.choiceBox.addItem(Combo_text, Combo_data)
            
            self.parent.Info_box.ensureCursorVisible()
            self.parent.Info_box.insertPlainText("\n \u2022 Edited stack : achieved.")   
            
            self.parent.displayExpStack(self.parent.Current_stack)
            
            self.close()
        
    def displayExpStack(self, series):
        self.expSeries.ui.histogram.hide()
        self.expSeries.ui.roiBtn.hide()
        self.expSeries.ui.menuBtn.hide()
        
        self.expSeries.addItem(self.crosshair_v1, ignoreBounds=True)
        self.expSeries.addItem(self.crosshair_h1, ignoreBounds=True) 
        
        view = self.expSeries.getView()
        state = view.getState()        
        self.expSeries.setImage(series) 
        view.setState(state)
        
        view.setBackgroundColor(self.parent.color1)
        ROIplot = self.expSeries.getRoiPlot()
        ROIplot.setBackground(self.parent.color1)
        
        font=QtGui.QFont('Noto Sans Cond', 9)
        ROIplot.getAxis("bottom").setTextPen('k') # Apply size of the ticks label
        ROIplot.getAxis("bottom").setTickFont(font)
        
        self.expSeries.timeLine.setPen(color=self.parent.color3, width=15)
        self.expSeries.frameTicks.setPen(color=self.parent.color1, width=5)
        self.expSeries.frameTicks.setYRange((0, 1))

        s = self.expSeries.ui.splitter
        s.handle(1).setEnabled(True)
        s.setStyleSheet("background: 5px white;")
        s.setHandleWidth(5)    
    
    def mouseMoved(self, e):
        pos = e[0]

        if self.expSeries.view.sceneBoundingRect().contains(pos):

            item = self.expSeries.view
            mousePoint = item.mapSceneToView(pos) 
                 
            self.crosshair_v1.setPos(mousePoint.x())
            self.crosshair_h1.setPos(mousePoint.y())

        try:
            self.x = int(mousePoint.x())
            self.y = int(mousePoint.y())
            
            if self.x >= 0 and self.y >= 0 and self.x < len(self.expStack[0, :, 0])and self.y < len(self.expStack[0, 0, :]):
                self.cursor_label.setText(str(self.x) + " ; " + str(self.y))
        except:
            pass

    def mouseClick(self, e):
        pos = e[0]
        
        fromPosX = pos.scenePos()[0]
        fromPosY = pos.scenePos()[1]
        
        posQpoint = QtCore.QPointF()
        posQpoint.setX(fromPosX)
        posQpoint.setY(fromPosY)

        if self.expSeries.view.sceneBoundingRect().contains(posQpoint):
                
            item = self.expSeries.view
            mousePoint = item.mapSceneToView(posQpoint) 

            self.crosshair_v1.setPos(mousePoint.x())
            self.crosshair_h1.setPos(mousePoint.y())
                 
            self.x = int(mousePoint.x())
            self.y = int(mousePoint.y())


# -*- coding: utf-8 -*-
"""
Created on Thu Sep  5 15:18:21 2024

@author: glhote1
"""

#%% Imports

import os
from os.path import abspath
import sys
from inspect import getsourcefile

import cv2
import numpy as np

from inichord import general_functions as gf
import tifffile as tf

import pyqtgraph as pg
from PyQt5.QtWidgets import QApplication, QMessageBox
from PyQt5 import QtCore, QtGui
from PyQt5.QtGui import QPixmap

path2thisFile = abspath(getsourcefile(lambda:0))
uiclass, baseclass = pg.Qt.loadUiType(os.path.dirname(path2thisFile) + "/Contour_map_ui_01.ui")

class MainWindow(uiclass, baseclass):
    def __init__(self,parent):
        super().__init__()

        self.setupUi(self)
        self.parent = parent
        
        self.OpenData.clicked.connect(self.loaddata) # Load data
        self.Compute_bttn.clicked.connect(self.compute_map) # Generate the contour map
        self.Push_validate.clicked.connect(self.extract_data)
        
        self.radius = self.Radius_box.value()
        self.threshold = self.Threshold_box.value()
        self.blur = int(self.Blur_box.currentText())
        self.sobel = int(self.Sobel_box.currentText())
        
        self.Radius_box.valueChanged.connect(self.get_modifications) # modification connexion (espacially to help batch processing GUI)
        self.Threshold_box.valueChanged.connect(self.get_modifications) # modification connexion (espacially to help batch processing GUI)
        self.Blur_box.activated.connect(self.get_modifications) # modification connexion (espacially to help batch processing GUI)
        self.Sobel_box.activated.connect(self.get_modifications) # modification connexion (espacially to help batch processing GUI)
        
        self.setWindowIcon(QtGui.QIcon('icons/filter_icon.png')) 
        self.defaultIV() # Default ImageView (when no image)
        
        try: # if data is imported from the main GUI
            self.expStack = parent.Current_stack
            self.displayExpStack(self.expStack)
        except:
            pass
        
        app = QApplication.instance()
        screen = app.screenAt(self.pos())
        geometry = screen.availableGeometry()
        
        # Position (self.move) and size (self.resize) of the main GUI on the screen
        self.move(int(geometry.width() * 0.1), int(geometry.height() * 0.1))
        self.resize(int(geometry.width() * 0.8), int(geometry.height() * 0.6))
        self.screen = screen

    def loaddata(self):
        StackLoc, StackDir = gf.getFilePathDialog("Import an image series") # Image importation
        
        self.expStack = [] # Initialization of the variable
        self.expStack = tf.TiffFile(StackLoc[0]).asarray() # Import the unit 2D array

        self.expStack = np.flip(self.expStack, 1)
        self.expStack = np.rot90(self.expStack, k=1, axes=(2, 1))

        self.displayExpStack(self.expStack)

    def get_modifications(self):
        self.radius = self.Radius_box.value()
        self.threshold = self.Threshold_box.value()
        self.blur = int(self.Blur_box.currentText())
        self.sobel = int(self.Sobel_box.currentText())

    def compute_map(self):
        self.working_stack = np.copy(self.expStack)
        
        self.progressBar.setValue(0) # Set the initial value of the Progress bar at 0
        self.progressBar.setRange(0, len(self.working_stack)-1) 
        
        # Application of the remove outlier step
        for i in range(0,len(self.expStack[:, 0, 0])): # Applique pour chaque slice les paramètres du remove outlier
        
            QApplication.processEvents()    
            self.ValSlice = i
            self.progression_bar()
        
            _, self.working_stack[i, :, :] =  gf.remove_outliers(self.expStack[i,:,:], self.radius, self.threshold)
            if self.blur != 0 :
                self.working_stack[i,:,:] =cv2.GaussianBlur(self.working_stack[i,:,:],(self.blur,self.blur),0)
            if self.sobel != 0 :
                self.grad_x = cv2.Sobel(self.working_stack[i,:,:],cv2.CV_32F,1,0,ksize=self.sobel)
                self.grad_y = cv2.Sobel(self.working_stack[i,:,:],cv2.CV_32F,0,1,ksize=self.sobel)
            
                self.working_stack[i,:,:] = cv2.addWeighted(np.absolute(self.grad_x), 0.5, np.absolute(self.grad_y), 0.5, 0)
        
        self.contour_map = np.sum(self.working_stack,0)
        self.contour_map = (self.contour_map - np.min(self.contour_map)) / (np.max(self.contour_map) - np.min(self.contour_map))

        self.displayres(self.contour_map)
        
    def extract_data(self):
        self.parent.contour_map = np.copy(self.contour_map) # Copy in the main GUI
        self.parent.StackList.append(self.contour_map) # Add the data in the stack list
        
        Combo_text = '\u2022 Contour map'
        Combo_data = self.contour_map
        self.parent.choiceBox.addItem(Combo_text, Combo_data) # Add the data in the QComboBox

        self.parent.displayDataview(self.parent.contour_map) # Display the labeled grain
        self.parent.choiceBox.setCurrentIndex(self.parent.choiceBox.count() - 1) # Show the last data in the choiceBox QComboBox

        self.parent.Info_box.ensureCursorVisible()
        self.parent.Info_box.insertPlainText("\n \u2022 Contour map.") 
        
        self.parent.Save_button.setEnabled(True)
        self.parent.Reload_button.setEnabled(True)
        self.parent.choiceBox.setEnabled(True)
        
        self.close()
        
    def progression_bar(self): # Function for the ProgressBar uses
        self.prgbar = self.ValSlice
        self.progressBar.setValue(self.prgbar)

    def defaultIV(self):
        # image series
        self.expSeries.clear()
        self.expSeries.ui.histogram.hide()
        self.expSeries.ui.roiBtn.hide()
        self.expSeries.ui.menuBtn.hide()
        
        view = self.expSeries.getView()
        view.setBackgroundColor(self.parent.color1)
        
        ROIplot = self.expSeries.getRoiPlot()
        ROIplot.setBackground(self.parent.color1)
        
        # Result map
        self.Resmap.clear()
        self.Resmap.ui.histogram.hide()
        self.Resmap.ui.roiBtn.hide()
        self.Resmap.ui.menuBtn.hide()
        
        view = self.Resmap.getView()
        view.setBackgroundColor(self.parent.color1)
        
        ROIplot = self.Resmap.getRoiPlot()
        ROIplot.setBackground(self.parent.color1)

    def displayExpStack(self, Series):
        self.expSeries.ui.histogram.hide()
        self.expSeries.ui.roiBtn.hide()
        self.expSeries.ui.menuBtn.hide()
        
        view = self.expSeries.getView()
        state = view.getState()        
        self.expSeries.setImage(Series) 
        view.setState(state)
        
        view.setBackgroundColor(self.parent.color1)
        ROIplot = self.expSeries.getRoiPlot()
        ROIplot.setBackground(self.parent.color1)
        
        font=QtGui.QFont('Noto Sans Cond', 9)
        ROIplot.getAxis("bottom").setTextPen('k') # Apply size of the ticks label
        ROIplot.getAxis("bottom").setTickFont(font)
        
        self.expSeries.timeLine.setPen(color=self.parent.color3, width=15)
        self.expSeries.frameTicks.setPen(color=self.parent.color1, width=5)
        self.expSeries.frameTicks.setYRange((0, 1))

        s = self.expSeries.ui.splitter
        s.handle(1).setEnabled(True)
        s.setStyleSheet("background: 5px white;")
        s.setHandleWidth(5) 
        
    def displayres(self, series):
        self.Resmap.ui.histogram.hide()
        self.Resmap.ui.roiBtn.hide()
        self.Resmap.ui.menuBtn.hide()
        
        view = self.Resmap.getView()
        state = view.getState()        
        self.Resmap.setImage(series) 
        view.setState(state)
        view.setBackgroundColor(self.parent.color1)
        
        self.Resmap.autoRange()



# -*- coding: utf-8 -*-
"""
Created on Wed Nov 22 23:26:20 2023

@author: clanglois1
"""
import os

from inspect import getsourcefile
from os.path import abspath

import numpy as np
import tifffile as tf
import time
import cv2
import largestinteriorrectangle as lir

import pyqtgraph as pg
from PyQt5.QtWidgets import QApplication
from PyQt5 import QtGui
from PyQt5.QtWidgets import QMessageBox
from PyQt5.QtGui import QPixmap

from inichord import general_functions as gf

from skimage import exposure
from scipy import ndimage as ndi

path2thisFile = abspath(getsourcefile(lambda:0))
uiclass, baseclass = pg.Qt.loadUiType(os.path.dirname(path2thisFile) + "/Stitching_v4.ui")

class MainWindow(uiclass, baseclass):
    def __init__(self,parent):
        super().__init__()
        
        self.setWindowIcon(QtGui.QIcon('icons/Stitch_icon.png'))
        
        self.setupUi(self)
        self.parent = parent

        self.x = 0
        self.y = 0
    
        self.defaultIV() # Hide the PlotWidget until a data has been loaded
        
        # Buttons disabled until first importation
        self.KADBox.setEnabled(False)
        self.col_spinBox.setEnabled(False)
        self.row_spinBox.setEnabled(False)
        self.Range_Edit.setEnabled(False)
        self.Tresh_spinBox.setEnabled(False)
        self.full_range.setEnabled(False)
        self.ComputeClass_bttn.setEnabled(False)
        self.Push_export.setEnabled(False)
        self.Save_bttn.setEnabled(False)
        
        self.OpenData.clicked.connect(self.loaddata) # Open the KAD maps
        self.ComputeClass_bttn.clicked.connect(self.Stitching) # Run the stitching program
        self.Push_export.clicked.connect(self.export_data)
        self.Save_bttn.clicked.connect(self.Save_results) # Save maps and informations
        
        self.KADBox.valueChanged.connect(self.Change_KAD_display)
        self.col_spinBox.valueChanged.connect(self.set_up)
        self.row_spinBox.valueChanged.connect(self.set_up)
        self.flag_KAD = False
        self.flag_affine = True # Affine transformation used as default 
        
        self.col_nbr = self.col_spinBox.value()
        self.row_nbr = self.row_spinBox.value()
        
        self.val_tresh = self.Tresh_spinBox.value()
        self.Tresh_spinBox.valueChanged.connect(self.Change_treshold)
        
        self.Range_Edit.setText("0")
        self.changeRange() # Modify the searching range 
        self.Range_Edit.editingFinished.connect(self.changeRange) # Modify the searching range 
        
        app = QApplication.instance()
        screen = app.screenAt(self.pos())
        geometry = screen.availableGeometry()
        
        # Control window position and dimensions
        self.move(int(geometry.width() * 0.05), int(geometry.height() * 0.05))
        self.resize(int(geometry.width() * 0.9), int(geometry.height() * 0.6))
        self.screen = screen
        
        # Icons sizes management for QMessageBox
        self.pixmap = QPixmap("icons/Stitch_icon.png")
        self.pixmap = self.pixmap.scaled(100, 100)

#%% Functions
    def data_choice(self): # Allow to apply other treatment depending if the data is a KAD one
        msg = QMessageBox.question(self, 'Image stitching', 'Is it a KAD data ?')
        if msg == QMessageBox.Yes:
            self.flag_KAD = True
        if msg == QMessageBox.No:
            self.flag_KAD = False

    def loaddata(self): # Must be a sequence of 2D array
        self.StackLoc, self.StackDir = gf.getFilePathDialog("data to be stitched (*.tiff)") 
        
        checkimage = tf.TiffFile(self.StackLoc[0]).asarray() # Check for dimension. If 2 dimensions : 2D array. If 3 dimensions : stack of images
        
        if checkimage.ndim != 2: # Check if the data is not a 2D array
            self.parent.popup_message("2D stitching","Please import a sequence of 2D arrays",'icons/Stitch_icon.png')
            return

        else:
            self.data_choice()
    
            self.KAD_base = []
            
            for i in range(0,len(self.StackLoc)): # Create a list of 2D array 
                Var = tf.TiffFile(self.StackLoc[i]).asarray()
                self.KAD_base.append(Var)
                
            for i in range(len(self.KAD_base)): 
                mask = np.isnan(self.KAD_base[i])
                self.KAD_base[i][mask] = np.interp(np.flatnonzero(mask), np.flatnonzero(~mask), self.KAD_base[i][~mask]) # Allow to convert NaN value with the nearest good value
                
                self.KAD_base[i] = (self.KAD_base[i] - np.min(self.KAD_base[i])) / (np.max(self.KAD_base[i]) - np.min(self.KAD_base[i])) # Normalization step
                self.KAD_base[i] = exposure.equalize_adapthist(self.KAD_base[i], kernel_size=None, clip_limit=0.01, nbins=256) # CLAHE step
    
                if self.KAD_base[i].dtype != 'uint8': # If data is not a 8bits one, then the conversion is apply
                    self.KAD_base[i] = gf.convertToUint8(self.KAD_base[i])
                
                Search0 = np.where(self.KAD_base[i] == 0.0) # Replace 0.0 by 1 for final crop efficiency
                self.KAD_base[i][Search0] = 1
                
            self.KAD_base_display = self.KAD_base.copy() 
            
            for i in range(len(self.KAD_base_display)): # Flip and rotation for display
                self.KAD_base_display[i] = np.flip(self.KAD_base_display[i], 0)
                self.KAD_base_display[i] = np.rot90(self.KAD_base_display[i], k=1, axes=(1, 0))
            
            self.displayExpKAD(self.KAD_base_display[0])
            self.InfoValues_label.setText(str(self.KAD_base_display[0].shape))
    
            # Buttons disabled until first importation
            self.KADBox.setEnabled(True)
            self.col_spinBox.setEnabled(True)
            self.row_spinBox.setEnabled(True)
            self.Range_Edit.setEnabled(True)
            self.Tresh_spinBox.setEnabled(True)
            self.full_range.setEnabled(True)
            self.ComputeClass_bttn.setEnabled(True)
            
        range_step = []
        for i in range(0,len(self.KAD_base)):
            range_step.append(np.mean(self.KAD_base[i].shape[1]))

        auto_val_range = int(np.round(np.mean(range_step)*0.3)) # 30% of the mean dimensions of images

        self.Range_Edit.setText(str(auto_val_range))
        self.val_range = auto_val_range
        
    def set_up(self):
        # Here, nbr of column and nbr of raw are defined. self.Super_listing is create.
        # it allow to work row by row for the next computation step
        self.col_nbr = self.col_spinBox.value()
        self.row_nbr = self.row_spinBox.value()

        self.Super_listing = []
        step = 0

        for i in range(0,self.row_nbr):
            self.Super_listing.append(self.KAD_base[step:self.col_nbr + step])
            
            step = step + self.col_nbr
            
        self.Super_listing = [x for x in self.Super_listing if x != []]      
    
    def Change_KAD_display(self): # Visualization of the different 2D array 
        self.KADBox.setRange(0,len(self.KAD_base)-1)
        self.KADBox.setSingleStep(1)
        self.Value = self.KADBox.value()
        
        self.displayExpKAD(self.KAD_base_display[self.Value])
        self.InfoValues_label.setText(str(self.KAD_base_display[self.Value].shape))

    def make_transfo_choice(self): # Choice of the transformation to be used to stitch images together
        if self.transfo_choice == 'Translation':
            self.flag_translation = True
            self.flag_affine = False
            self.flag_homo = False
        
        elif self.transfo_choice == 'Affine':
            self.flag_translation = False
            self.flag_affine = True
            self.flag_homo = False
            
        elif self.transfo_choice == 'Homography':
            self.flag_translation = False
            self.flag_affine = False
            self.flag_homo = True
            
    def Stitching(self): # Stitch images together   
        self.transfo_choice = self.Transfo_box.currentText() # Extract transformation choice
        self.make_transfo_choice() # Define which transformation must be used
        
        self.nbr = self.col_nbr * self.row_nbr # Nbr of total 2D array
        
        self.prgbar = 0 # Progress bar initial value
        self.progressBar.setValue(self.prgbar)
        self.progressBar.setRange(0, self.nbr-1)
        self.increment = 0

        try:
            self.set_up()
            self.res = []

            if self.col_nbr > 1: # If there are more than 1 column
                for i in range(0,len(self.Super_listing)):
                    for j in range(len(self.Super_listing[0])-1,0,-1):
                        # Find matches between images 
                        M = self.find_matches(self.Super_listing[i][j],self.Super_listing[i][j-1],direction = "horizontal")
                        # Apply transformation matrix M
                        dst = self.horizontal_stitching(self.Super_listing[i][j],self.Super_listing[i][j-1],M)
                        # Stitch the two images together
                        self.Super_listing[i][j] = self.Super_listing[i][0:j]
                        self.Super_listing[i][j-1] = dst
                        
                        QApplication.processEvents()    
                        self.increment = self.increment + 1
                        self.ValSlice = self.increment
                        self.progression_bar()
                        
                        # j is modified because the lenght of Super listing is evolving at each step
                        # j = len(self.Super_listing[i])+1
                        j = len(self.Super_listing[i])-1
                        
                        # The twoD stitching is cropped
                        self.cropped_dst = self.cropping_step(dst)
                        
                    # The twoD stitching is stored to be used after 
                    self.res.append(self.cropped_dst)
                    
            else : # If there is only 1 column, self.res is a copy of self.KAD_base 
                self.res = self.KAD_base.copy()
        
            if self.row_nbr > 1: # If the nbr of row is higher than 1
                for i in range(len(self.res)-1,0,-1):
                    # Find matches between images 
                    M = self.find_matches(self.res[i],self.res[i-1],direction = "vertical")
                    # Apply transformation matrix M
                    dst = self.vertical_stitching(self.res[i],self.res[i-1],M)
                    # Stitch the two images together
                    self.res[i] = self.res[0:i]
                    self.res[i-1] = dst
                    
                    QApplication.processEvents()    
                    self.increment = self.increment + 1
                    self.ValSlice = self.increment
                    self.progression_bar()
                    
                    # i is modified because the lenght of self.res is evolving at each step
                    i = len(self.res[i])-1
                    
                    # The twoD stitching is cropped
                    self.cropped_dst = self.cropping_step(dst)
            
            # Apply flip and rotation for display of the stitched 2D arrays
            self.displayed_dst = np.copy(self.cropped_dst)
            self.displayed_dst = np.flip(self.displayed_dst, 0)
            self.displayed_dst = np.rot90(self.displayed_dst, k=1, axes=(1, 0))
                    
            self.displayStitch(self.displayed_dst)
            
            self.Push_export.setEnabled(True) # Allow to export to main GUI
            self.Save_bttn.setEnabled(True) # Allow to save data in a folder
            
        except:
            self.parent.popup_message("2D stitching","Stitch failed. Please check that the [columns - rows] information match the imported data.",'icons/Stitch_icon.png')
            return

    def find_matches(self,data1, data2, direction = "horizontal"):
        sift = cv2.SIFT_create()
        
        # Allow to specifiy if the descriptor determination must be apply on the whole images or only a part of it
        if self.full_range.isChecked():
            self.val_range = len(data2[0])

        # create a mask image filled with zeros, the size of original image
        mask = np.zeros(data1.shape[:2], dtype=np.uint8)
        mask2 = np.zeros(data2.shape[:2], dtype=np.uint8)
        
        # Allow to specifiy the range of search to consider the fact that the images are vertical or horizontal
        if direction == 'horizontal':
            cv2.rectangle(mask, (0,0), (self.val_range,len(data1)), (255), thickness = -1)
            cv2.rectangle(mask2, (len(data2[0])-self.val_range,0), (len(data2[0]),len(data2)), (255), thickness = -1)
        if direction == 'vertical':
            cv2.rectangle(mask, (0,0), (len(data1[0]),self.val_range), (255), thickness = -1)
            cv2.rectangle(mask2, (len(data2[0]),len(data2)), (0,len(data2)-self.val_range), (255), thickness = -1)
    
        # Extract descriptor in the two images
        kp1, des1 = sift.detectAndCompute(data1,mask)
        kp2, des2 = sift.detectAndCompute(data2,mask2)
        
        # Look for matches
        match = cv2.BFMatcher()
        matches = match.knnMatch(des1,des2,k=2)
        
        # Matches filtering in order to keep only the best pairs
        good = []
        for m,n in matches:
            if m.distance < self.val_tresh*n.distance:
                good.append(m)
        
        MIN_MATCH_COUNT = 10 # Minimum nbr of descriptor needed to compute
        if len(good) > MIN_MATCH_COUNT:
            src_pts = np.float32([ kp1[m.queryIdx].pt for m in good ]).reshape(-1,1,2)
            dst_pts = np.float32([ kp2[m.trainIdx].pt for m in good ]).reshape(-1,1,2)
        
            # Look for transformaiton matrix between the two set of descriptors
            if self.flag_translation == True : 
                M, mask = cv2.findHomography(src_pts, dst_pts, cv2.RANSAC, 5.0) #Homography
                
                M[1:3,0] = 0
                M[2,1] = 0
                M[0,1] = 0
                
            elif self.flag_affine == True : 
                M, mask = cv2.estimateAffinePartial2D(src_pts, dst_pts) #Affine
            elif self.flag_homo == True : 
                M, mask = cv2.findHomography(src_pts, dst_pts, cv2.RANSAC, 5.0) #Homography
        
        else:
            self.parent.popup_message("2D stitching","Not enough descriptor...",'icons/Stitch_icon.png')
            return
            
        return M # Return the matrix of transformation between the 2 images

    def horizontal_stitching(self,data1,data2,M):
        # Specify the initial shape of dst as a function of the transformation (translation/affine: warpAffine ; else: warpPerspective)
        if self.flag_affine == True : 
            dst = cv2.warpAffine(data1,M,(data2.shape[1] + data1.shape[1], data2.shape[0]))
        else :
            dst = cv2.warpPerspective(data1,M,(data2.shape[1] + data1.shape[1], data2.shape[0]))
        
        dst[0:data2.shape[0],0:data2.shape[1]] = data2
        
        return dst
    
    def vertical_stitching(self,data1,data2,M):   
        # Specify the initial shape of dst as a function of the transformation (translation/affine: warpAffine ; else: warpPerspective)
        if self.flag_affine == True : 
            dst = cv2.warpAffine(data1,M,(data2.shape[1] + data1.shape[1], data2.shape[0] + data1.shape[0]))
        else:
            dst = cv2.warpPerspective(data1,M,(data2.shape[1] + data1.shape[1], data2.shape[0] + data1.shape[0]))

        dst[0:data2.shape[0],0:data2.shape[1]] = data2
        
        return dst
        
    def cropping_step(self,dst): # Crop images to remove black border 
        Min_Aligned_stack = self.Mask_min(dst,0)  
        Min_Aligned_stack = Min_Aligned_stack.astype("bool")
        Min_Aligned_stack = ndi.binary_fill_holes(Min_Aligned_stack).astype("bool")
    
        Selection = lir.lir(Min_Aligned_stack) # array([2, 2, 4, 7])
        Cropped_dst = dst[Selection[1]:Selection[3],Selection[0]:Selection[2]]  
        
        return Cropped_dst
    
    def Mask_min(self,Min_Aligned_stack, threshold):
        Mask = np.zeros((len(Min_Aligned_stack),len(Min_Aligned_stack[0])))
        Mask[Min_Aligned_stack > threshold] = 1
        return Mask
    
    def changeRange(self): # Take into account the searching range modification
        self.val_range = int(self.Range_Edit.text())
        self.full_range.setChecked(False)
        
    def Change_treshold(self): # Take into account the descriptor filtering threshold
        self.val_tresh = self.Tresh_spinBox.value()
    
    def progression_bar(self): # Fonction relative à la barre de progression
        self.prgbar = self.ValSlice
        self.progressBar.setValue(self.prgbar)

    def Save_results(self):
        ti = time.strftime("%Y-%m-%d__%Hh-%Mm-%Ss") # Absolute time 
        
        directory = "Stitched_images_" + ti # Name of the main folder
        PathDir = os.path.join(self.StackDir, directory)  # where to create the main folder
        os.mkdir(PathDir)  # Create main folder

        # Images saving step
        tf.imwrite(PathDir + '/Cropped_stitched_img.tiff', np.rot90(np.flip(self.displayed_dst, 0), k=1, axes=(1, 0)).astype('float32')) 

        # Information (.TXT) step
        with open(PathDir + '\Image stitching.txt', 'w') as file:
            file.write("KAD data: " + str(self.flag_KAD))
            file.write("\nNumber of images: " + str(len(self.KAD_base)))
            file.write("\nNumber of row: " + str(self.row_nbr))   
            file.write("\nNumber of column: " + str(self.col_nbr))  
            if self.full_range.isChecked():
                file.write("\nSearching range (pxls): full images")   
            else:
                file.write("\nSearching range (pxls): "+ str(self.val_range))  
            file.write("\nTransformation: " + str(self.val_tresh)) 
            file.write("\nThreshold: " + str(self.transfo_choice)) 

        # Finished message
        self.parent.popup_message("2D stitching","Saving process is over.",'icons/Stitch_icon.png')

    def export_data(self): # Push stitched image in the main GUI
        if self.flag_KAD == True: # If stitching has been applied on KAD data 
            self.parent.flag_stitchKAD = True
            
            self.parent.KAD = np.copy(self.displayed_dst) # Copy in the main GUI
            self.parent.StackList.append(self.displayed_dst) # Add the data in the stack list
            
            self.parent.StackDir = self.StackDir
            
            self.parent.displayDataview(self.parent.KAD) # Display the labeled grain
            self.parent.choiceBox.setCurrentIndex(self.parent.choiceBox.count() - 1) # Show the last data in the choiceBox QComboBox
        else :
            self.parent.flag_stitchKAD = False
            
            self.parent.Contour_map = np.copy(self.displayed_dst) # Copy in the main GUI
            self.parent.StackList.append(self.displayed_dst) # Add the data in the stack list
            
            self.parent.StackDir = self.StackDir
            
            self.parent.displayDataview(self.parent.Contour_map) # Display the labeled grain
            self.parent.choiceBox.setCurrentIndex(self.parent.choiceBox.count() - 1) # Show the last data in the choiceBox QComboBox
        
        Combo_text = '\u2022 Stitched map'
        Combo_data = self.displayed_dst
        self.parent.choiceBox.addItem(Combo_text, Combo_data) # Add the data in the QComboBox

        self.parent.Info_box.ensureCursorVisible()
        self.parent.Info_box.insertPlainText("\n \u2022 Stitched map.") 
             
        self.parent.Save_button.setEnabled(True)
        self.parent.Reload_button.setEnabled(True)
        self.parent.choiceBox.setEnabled(True)
        
        # Finished message
        self.parent.popup_message("2D stitching","Stitched array has been exported to the main GUI.",'icons/Stitch_icon.png')
    
    def displayExpKAD(self, series): # Display of initial KAD maps
        self.KADSeries.ui.histogram.hide()
        self.KADSeries.ui.roiBtn.hide()
        self.KADSeries.ui.menuBtn.hide()
        
        view = self.KADSeries.getView()
        state = view.getState()        
        self.KADSeries.setImage(series) 
        view.setState(state)
        view.setBackgroundColor(self.parent.color1)
        
        self.KADSeries.autoRange()
        
    def displayStitch(self, series): # Display of initial KAD map
        self.StitchSeries.ui.histogram.show()
        self.StitchSeries.ui.roiBtn.hide()
        self.StitchSeries.ui.menuBtn.hide()
        
        view = self.StitchSeries.getView()
        state = view.getState()        
        self.StitchSeries.setImage(series) 
        view.setState(state)
        view.setBackgroundColor(self.parent.color1)
        
        self.StitchSeries.autoRange()
        
        histplot = self.StitchSeries.getHistogramWidget()
        histplot.setBackground(self.parent.color1)
        
        histplot.region.setBrush(pg.mkBrush(self.parent.color5 + (120,)))
        histplot.region.setHoverBrush(pg.mkBrush(self.parent.color5 + (60,)))
        histplot.region.pen = pg.mkPen(self.parent.color5)
        histplot.region.lines[0].setPen(pg.mkPen(self.parent.color5, width=2))
        histplot.region.lines[1].setPen(pg.mkPen(self.parent.color5, width=2))
        histplot.fillHistogram(color = self.parent.color5)        
        histplot.autoHistogramRange()       

    def defaultIV(self):
        self.KADSeries.ui.histogram.hide()
        self.KADSeries.ui.roiBtn.hide()
        self.KADSeries.ui.menuBtn.hide()
        
        view = self.KADSeries.getView()
        view.setBackgroundColor(self.parent.color1)
        
        ROIplot = self.KADSeries.getRoiPlot()
        ROIplot.setBackground(self.parent.color1)
        
        self.StitchSeries.ui.histogram.hide()
        self.StitchSeries.ui.roiBtn.hide()
        self.StitchSeries.ui.menuBtn.hide()
        
        view = self.StitchSeries.getView()
        view.setBackgroundColor(self.parent.color1)


# -*- coding: utf-8 -*-
"""
Created on Tue Nov 14 17:05:45 2023

@author: glhote1
"""

## Ce programme à pour objectif de permettre de sortir rapidement un KAM de distance d'une stack alignée
'A noter, la valeur de la carte de KAD ne sera pas nécessairement comprise entre 0 et 1 sauf pour le kernel "Homogeneous".'

# Importation et fonctions
import numpy as np

def centeredEuclidianNorm(prof, ax = 1):
    mean =np.mean(prof, axis = ax, keepdims=True)
    prof = prof - mean
    del mean
    norm = np.linalg.norm(prof, axis = ax, keepdims=True)
    prof = prof / norm
    
    return prof

def KAM_rapide(rawImage_1): 
    
    Pr_length = len(rawImage_1)
    
    rawImage_roll1 = np.roll(rawImage_1,1, axis = 1)
    rawImage_roll2 = np.roll(rawImage_1,1, axis = 2)
    rawImage_roll3 = np.roll(rawImage_1,-1, axis = 1)
    rawImage_roll4 = np.roll(rawImage_1,-1, axis = 2)
    rawImage_roll5 = np.roll(rawImage_1,(1,1), axis = (1,2))
    rawImage_roll6 = np.roll(rawImage_1,(1,-1), axis = (1,2))
    rawImage_roll7 = np.roll(rawImage_1,(-1,1), axis = (1,2))
    rawImage_roll8 = np.roll(rawImage_1,(-1,-1), axis = (1,2))
    
    # Reshape des profils
    
    rawImage_reshape = np.reshape(np.swapaxes(rawImage_1,0,2),(-1,Pr_length))
    rawImage_roll1_reshape = np.reshape(np.swapaxes(rawImage_roll1,0,2),(-1,Pr_length)).T
    rawImage_roll2_reshape = np.reshape(np.swapaxes(rawImage_roll2,0,2),(-1,Pr_length)).T
    rawImage_roll3_reshape = np.reshape(np.swapaxes(rawImage_roll3,0,2),(-1,Pr_length)).T
    rawImage_roll4_reshape = np.reshape(np.swapaxes(rawImage_roll4,0,2),(-1,Pr_length)).T
    rawImage_roll5_reshape = np.reshape(np.swapaxes(rawImage_roll5,0,2),(-1,Pr_length)).T
    rawImage_roll6_reshape = np.reshape(np.swapaxes(rawImage_roll6,0,2),(-1,Pr_length)).T
    rawImage_roll7_reshape = np.reshape(np.swapaxes(rawImage_roll7,0,2),(-1,Pr_length)).T
    rawImage_roll8_reshape = np.reshape(np.swapaxes(rawImage_roll8,0,2),(-1,Pr_length)).T
    
    # Calcul de la distance entre la carte de rÃ©fÃ©rence et chaques cartes "rolls"
    
    Matmul1 = []
    Matmul2 = []
    Matmul3 = []
    Matmul4 = []
    Matmul5 = []
    Matmul6 = []
    Matmul7 = []
    Matmul8 = []
    
    for i in range(0,len(rawImage_reshape)):
        Var_1 = np.matmul(rawImage_reshape[i,:],rawImage_roll1_reshape[:,i])
        Var_2 = np.matmul(rawImage_reshape[i,:],rawImage_roll2_reshape[:,i])
        Var_3 = np.matmul(rawImage_reshape[i,:],rawImage_roll3_reshape[:,i])
        Var_4 = np.matmul(rawImage_reshape[i,:],rawImage_roll4_reshape[:,i])
        Var_5 = np.matmul(rawImage_reshape[i,:],rawImage_roll5_reshape[:,i])
        Var_6 = np.matmul(rawImage_reshape[i,:],rawImage_roll6_reshape[:,i])
        Var_7 = np.matmul(rawImage_reshape[i,:],rawImage_roll7_reshape[:,i])
        Var_8 = np.matmul(rawImage_reshape[i,:],rawImage_roll8_reshape[:,i])
        
        Matmul1.append(Var_1)
        Matmul2.append(Var_2)
        Matmul3.append(Var_3)
        Matmul4.append(Var_4)
        Matmul5.append(Var_5)
        Matmul6.append(Var_6)
        Matmul7.append(Var_7)
        Matmul8.append(Var_8)
    
    Matmul1 = np.vstack(Matmul1)
    Matmul2 = np.vstack(Matmul2)
    Matmul3 = np.vstack(Matmul3)
    Matmul4 = np.vstack(Matmul4)
    Matmul5 = np.vstack(Matmul5)
    Matmul6 = np.vstack(Matmul6)
    Matmul7 = np.vstack(Matmul7)
    Matmul8 = np.vstack(Matmul8)
    
    Matmul1 = np.reshape(Matmul1,(len(rawImage_1[0][0]),len(rawImage_1[0])))
    Matmul2 = np.reshape(Matmul2,(len(rawImage_1[0][0]),len(rawImage_1[0])))
    Matmul3 = np.reshape(Matmul3,(len(rawImage_1[0][0]),len(rawImage_1[0])))
    Matmul4 = np.reshape(Matmul4,(len(rawImage_1[0][0]),len(rawImage_1[0])))
    Matmul5 = np.reshape(Matmul5,(len(rawImage_1[0][0]),len(rawImage_1[0])))
    Matmul6 = np.reshape(Matmul6,(len(rawImage_1[0][0]),len(rawImage_1[0])))
    Matmul7 = np.reshape(Matmul7,(len(rawImage_1[0][0]),len(rawImage_1[0])))
    Matmul8 = np.reshape(Matmul8,(len(rawImage_1[0][0]),len(rawImage_1[0])))
    
    Distance_JDG = (Matmul1 + Matmul2 + Matmul3 + Matmul4 + Matmul5 + Matmul6 + Matmul7 + Matmul8)/(9)
    
    Distance_JDG = np.rot90(Distance_JDG, k=1, axes=(0, 1))
    Distance_JDG = np.flip(Distance_JDG)
    Distance_JDG = np.fliplr(Distance_JDG)
    Distance_JDG = 1-Distance_JDG
    
    Distance_JDG[1:-1,0] = Distance_JDG[1:-1,1] # Col gauche
    Distance_JDG[1:-1,-1] = Distance_JDG[1:-1,-2] # Col droite
    Distance_JDG[0,:] = Distance_JDG[1,:]
    Distance_JDG[-1,:] = Distance_JDG[-2,:]
  
    del(Var_1,Var_2,Var_3,Var_4,Var_5,Var_6,Var_7,Var_8)
    del(rawImage_roll1,rawImage_roll2,rawImage_roll3,rawImage_roll4,rawImage_roll5,rawImage_roll6,rawImage_roll7,rawImage_roll8)
    del(rawImage_roll1_reshape,rawImage_roll2_reshape,rawImage_roll3_reshape,rawImage_roll4_reshape,rawImage_roll5_reshape,rawImage_roll6_reshape,rawImage_roll7_reshape,rawImage_roll8_reshape)
    del(Matmul1,Matmul2,Matmul3,Matmul4,Matmul5,Matmul6,Matmul7,Matmul8)

    return Distance_JDG

def Divided_KAD(rawImage_1, divider = 20):
    Conca_KAD = []
    list_i = []
    
    height = len(rawImage_1[0]) # Hauteur d'une image
    rec = 3 # Nombre de pixels pour le recouvrement

    for i in range(1,divider):
        if height%i > rec+1 and height/i < 1000: # Critères : stacks de moins de 1000 pixels de haut, et dernière stack avec suffisament de pixel (rec+1)
            list_i.append(i) # Extraction dans la liste des diviseurs ceux qui sont possibles
            
    min_i = np.min(list_i) # Selection du plus petit possible pour diviser le moins possible
    h_prime = height//min_i # Hauteur des stacks

    for i in range(0,min_i):       
        var = rawImage_1[:,h_prime*i:h_prime*(i+1)+rec,:] # Extraction de chaque sous regions
        var_distance = KAM_rapide(var) # Calcul du KAD sur chaque sous regions

        Conca_KAD.append(var_distance) # Stockage dans une liste des différentes cartes
      
    var = rawImage_1[:,h_prime*min_i:,:] # Extraction de la dernière sous regions
    var_distance = KAM_rapide(var) # Calcul du KAD sur la dernière portion du stack
        
    Conca_KAD.append(var_distance) # Ajout du dernier KAD
    lim = int(rec/2)
    
    xtract = np.zeros((len(rawImage_1[0]),len(rawImage_1[0][0]))) # Creation du tableau de concatenation des KAD

    # Remplissage de la fin du tableau
    End_shape = Conca_KAD[-1].shape
    xtract[-End_shape[0]:,:] = Conca_KAD[-1]

    # Remplissage du tableau en partant de la fin 
    for i in range(min_i,0,-1):
        xtract[(i-1)*h_prime :i*h_prime + lim+1] = Conca_KAD[i-1][:-lim]
        
    return xtract



# -*- coding: utf-8 -*-
"""
Created on Thu Sep  5 15:18:21 2024

@author: glhote1
"""

#%% Imports

from inspect import getsourcefile
from os.path import abspath

from pyqtgraph.Qt import QtGui
import pyqtgraph as pg
from PyQt5.QtWidgets import QApplication

from inichord import general_functions as gf
import tifffile as tf
import numpy as np
import tkinter as tk
from tkinter import filedialog
import os
from scipy.fft import fft2, ifft2, fftshift

path2thisFile = abspath(getsourcefile(lambda:0))
uiclass, baseclass = pg.Qt.loadUiType(os.path.dirname(path2thisFile) + "/Extract_mosaic_ui_02.ui")

class MainWindow(uiclass, baseclass):
    def __init__(self,parent):
        super().__init__()

        self.setupUi(self)
        self.setWindowIcon(QtGui.QIcon('icons/Main_icon.png')) # Application of the main icon
        self.parent = parent

        self.OpenData.clicked.connect(self.loaddata) # Load data
        self.Xtract_series.clicked.connect(self.serie_extraction) # Extract the different series 
        self.Erroneous_bttn.clicked.connect(self.image_correction) # Replace image with problem
        self.Save_bttn.clicked.connect(self.save_data) # Save series in a given folder
        self.ProcessRef_bttn.clicked.connect(self.Ref_processing) # Process images references 
        
        self.defaultIV() # Default ImageView (when no image)
        
        self.flag_FFTCorrection = 0 # Initialization
        
        # disable buttons at the beginning
        self.nbrSeries.setEnabled(False)
        self.First_img.setEnabled(False)
        self.DriftCorr_choice.setEnabled(False)
        self.Xtract_series.setEnabled(False)
        self.Threshold.setEnabled(False)
        self.Erroneous_bttn.setEnabled(False)
        self.Save_bttn.setEnabled(False)
        
        app = QApplication.instance()
        screen = app.screenAt(self.pos())
        geometry = screen.availableGeometry()
        
        # Position (self.move) and size (self.resize) of the main GUI on the screen
        self.move(int(geometry.width() * 0.1), int(geometry.height() * 0.1))
        self.resize(int(geometry.width() * 0.8), int(geometry.height() * 0.6))
        self.screen = screen

    def Ref_processing(self):
        # Import reference images
        StackLoc, StackDir = gf.getFilePathDialog("Reference images") # Image importation
        self.image_ref = [] 
        
        for i in range(0,len(StackLoc)):
            Var = tf.TiffFile(StackLoc[i]).asarray() # Import the unit 2D array
            self.image_ref.append(Var) # Append every 2D array in a list named self.image_ref
            
        self.image_ref = np.dstack(self.image_ref) # Convert list of 3D array
        self.image_ref = np.swapaxes(self.image_ref, 0, 2) # Rearrangement of the axis
        self.image_ref = np.swapaxes(self.image_ref, 1, 2) # Rearrangement of the axis
            
        self.image_ref = self.image_ref[::-1]
        # Ask for the saving folder
        root = tk.Tk()
        root.withdraw()  # Masquer la fenêtre principale Tkinter
    
        # Ouvrir une boîte de dialogue pour choisir un dossier de sauvegarde
        dossier = filedialog.askdirectory(title="Choose a folder for reference images.")
        # Save in the folder with the good index

        for i in range(self.image_ref.shape[0]):
            # Générer le nom du fichier avec un numéro incrémenté
            chemin_complet = os.path.join(dossier, f"reference_{i}.tiff")
        
            # Sauvegarder le tableau 2D en tant que fichier TIFF
            tf.imwrite(chemin_complet, self.image_ref[i])
            
        # Finished message
        self.parent.popup_message("Extract mosaic","Reference images have been saved.",'icons/Main_icon.png')

    def loaddata(self):
        StackLoc, StackDir = gf.getFilePathDialog("Images") # Image importation
        
        self.image = [] # Initialization of the variable self.image (which will be the full image folder)
        
        self.progressBar.setValue(0) # Set the initial value of the Progress bar at 0
        self.progressBar.setRange(0, len(StackLoc)-1) 
        self.progressBar.setFormat("Loading images... %p%")

        for i in range(0,len(StackLoc)):
            Var = tf.TiffFile(StackLoc[i]).asarray() # Import the unit 2D array
            self.image.append(Var) # Append every 2D array in a list named self.image
            
            QApplication.processEvents()    
            self.ValSlice = i
            self.progression_bar()
            
        self.image = np.dstack(self.image) # Convert list of 3D array
        self.image = np.swapaxes(self.image, 0, 2) # Rearrangement of the axis
        self.image = np.swapaxes(self.image, 1, 2) # Rearrangement of the axis
        
        self.image_displayed = np.flip(self.image, 1) # Flip the array
        self.image_displayed = np.rot90(self.image_displayed, k=1, axes=(2, 1)) # Rotate the array
        
        self.displayExpStack(self.image_displayed) # Display the 3D array

        # enables buttons at the beginning
        self.nbrSeries.setEnabled(True)
        self.First_img.setEnabled(True)
        self.DriftCorr_choice.setEnabled(True)
        self.Xtract_series.setEnabled(True)
        
    def serie_extraction(self):
        self.series_nbr = self.nbrSeries.value() # Number of series involved
        self.First_image = self.First_img.value() # Number of the first image
        
        # To define if drift correction image has been acquired
        self.correction_choice = self.DriftCorr_choice.currentText()
        if self.correction_choice == "Yes":
            self.flag_corr = 1
        else :
            self.flag_corr = 0
            
        image_reduction = self.image[self.First_image-1:,:,:]
        self.images_dict = {} # Dictionnary creation

        if self.flag_corr == 1 :
            intervalle = self.series_nbr + 1
            indices_a_conserver = [i for i in range(image_reduction.shape[0]) if (i + 1) % intervalle != 0]
            image_reduction = image_reduction[indices_a_conserver]

        self.images_dict = {f"images_{i}": [] for i in range(self.series_nbr)}
            
        # Parcourir chaque image 2D dans le tableau 3D
        for index, image2D in enumerate(image_reduction):
            # Déterminer à quel groupe (liste) l'image doit être ajoutée
            groupe_index = index % self.series_nbr
            self.images_dict[f"images_{groupe_index}"].append(image2D)

        # Appliquer les opérations numpy à chaque liste du dictionnaire
        i= 0
        for key in self.images_dict:
            self.progressBar.setValue(0) # Set the initial value of the Progress bar at 0
            self.progressBar.setRange(0, len(self.images_dict)-1)
            self.progressBar.setFormat("Create series...")
            
            # Si la liste n'est pas vide
            if self.images_dict[key]:
                # Combiner les images de la liste le long du troisième axe avec dstack
                combined_image = np.dstack(self.images_dict[key])

                # Appliquer les deux swapaxes successifs
                transformed_image = np.swapaxes(combined_image, 0, 2)
                transformed_image = np.swapaxes(transformed_image, 1, 2)

                # Mettre à jour le dictionnaire avec l'image transformée
                self.images_dict[key] = transformed_image
            else:
                self.images_dict[key] = None  # Gérer les cas où il n'y a pas d'images dans le groupe

            QApplication.processEvents()    
            self.ValSlice = i
            self.progression_bar()
            
            i = i+1
        
        self.progressBar.setFormat("Series have been created")
        
        # Enables buttons
        self.Threshold.setEnabled(True)
        self.Erroneous_bttn.setEnabled(True)
        self.Save_bttn.setEnabled(True)

    def image_correction(self):
        self.Threshold_value = self.Threshold.value()
        self.Corrected_dict = {f"images_{i}": [] for i in range(self.series_nbr)}
        
        self.progressBar.setValue(0) # Set the initial value of the Progress bar at 0
        self.progressBar.setRange(0, len(self.images_dict)-1)
        self.progressBar.setFormat("Correct series... %p%")
        
        for i, nom in enumerate(list(self.images_dict.keys())):
            array_3d = self.images_dict[nom]
            distance = []
        
            # Comparer chaque image avec l'image de référence
            for j in range(0, array_3d.shape[0]):
                if j == 0:
                    image_reference = array_3d[j]
                else:
                    image_reference = array_3d[j-1]
                    
                decalage = self.Compute_shift(image_reference, array_3d[j])
                distance_decalage = np.linalg.norm(decalage)
                
                distance.append(distance_decalage)
        
            index = np.hstack(distance)
            index = index < self.Threshold_value
            
            # Parcourir l'array pour trouver les paires de 'False'
            for k in range(len(index) - 1):
                # Vérifier si deux éléments consécutifs sont 'False'
                if index[k] == False and index[k + 1] == False:       
                    array_3d[k] = array_3d[k - 1]
        
            self.Corrected_dict[f"images_{i}"].append(array_3d)
            
            QApplication.processEvents()    
            self.ValSlice = i
            self.progression_bar()

        self.progressBar.setFormat("Series have been corrected")
        self.flag_FFTCorrection = 1

    def Compute_shift(self,image1, image2):
        # Calculer la transformation de Fourier des deux images
        image1 = gf.convertToUint8(image1)
        image2 = gf.convertToUint8(image2)
        
        f_image1 = fft2(image1)
        f_image2 = fft2(image2)

        # Calculer la corrélation croisée en utilisant le produit conjugué
        produit_conjugue = f_image1 * np.conj(f_image2)
        cross_correlation = fftshift(ifft2(produit_conjugue))

        # Trouver le pic de la corrélation croisée pour déterminer le décalage
        decalage = np.unravel_index(np.argmax(np.abs(cross_correlation)), cross_correlation.shape)

        # Calculer le décalage relatif à partir du centre
        centre = np.array(image1.shape) // 2
        decalage = np.array(decalage) - centre

        return decalage

    def save_data(self):
        if self.flag_FFTCorrection == 1:
            self.sauvegarder_tableaux(self.Corrected_dict)
        elif self.flag_FFTCorrection == 0:
            self.sauvegarder_tableaux(self.images_dict)
        
    def sauvegarder_tableaux(self,tableaux):
        # Initialiser Tkinter
        root = tk.Tk()
        root.withdraw()  # Masquer la fenêtre principale Tkinter
    
        # Ouvrir une boîte de dialogue pour choisir un dossier de sauvegarde
        dossier = filedialog.askdirectory(title="Choose a folder for image series.")
    
        self.progressBar.setValue(0) # Set the initial value of the Progress bar at 0
        self.progressBar.setRange(0, len(self.images_dict)-1) 
        self.progressBar.setFormat("Save series...%p%")
    
        # Vérifier si un dossier a été sélectionné
        if dossier:
            # Récupérer les clés du dictionnaire et les inverser
            cles_inverses = list(reversed(list(tableaux.keys())))
    
            # Boucle pour chaque clé et tableau dans le dictionnaire avec indices inversés
            for i, nom in enumerate(cles_inverses, start=1):
                tableau = tableaux[nom]
                # Définir le chemin complet du fichier avec le nom de la clé et l'indice inversé
                chemin_complet = os.path.join(dossier, f"serie_{i}.tiff")
    
                # Sauvegarder le tableau 3D au format TIFF avec le type de données d'origine
                tf.imwrite(chemin_complet, tableau)
                
                QApplication.processEvents()    
                self.ValSlice = i
                self.progression_bar()
        else:
            print("Aucun dossier sélectionné. Annulation de la sauvegarde.")
            
        self.progressBar.setFormat("Series have been saved")

    def progression_bar(self): # Function for the ProgressBar uses
        self.prgbar = self.ValSlice
        self.progressBar.setValue(self.prgbar)
        
    def defaultIV(self):
        self.expSeries.clear()
        self.expSeries.ui.histogram.hide()
        self.expSeries.ui.roiBtn.hide()
        self.expSeries.ui.menuBtn.hide()
        
        view = self.expSeries.getView()
        view.setBackgroundColor(self.parent.color1)
        
        ROIplot = self.expSeries.getRoiPlot()
        ROIplot.setBackground(self.parent.color1)
        
    def displayExpStack(self, series):   
        self.expSeries.ui.histogram.hide()
        self.expSeries.ui.roiBtn.hide()
        self.expSeries.ui.menuBtn.hide()
        
        view = self.expSeries.getView() # Extract view
        state = view.getState() # Extract state
        self.expSeries.setImage(series) # Add the wanted data inside the ImageView
        view.setState(state)
        
        view.setBackgroundColor(self.parent.color1) # Define the background color
        ROIplot = self.expSeries.getRoiPlot() # Extract the ROI
        ROIplot.setBackground(self.parent.color1) # Define the ROI background color
        
        font=QtGui.QFont('Noto Sans Cond', 8) # Set fontsize
        ROIplot.getAxis("bottom").setTextPen('k') # Apply color of the ticks label
        ROIplot.getAxis("bottom").setTickFont(font) # Apply size of the ticks label
        
        self.expSeries.timeLine.setPen(color=self.parent.color3, width=15) # Define timeLine line color and width
        self.expSeries.frameTicks.setPen(color=self.parent.color1, width=5) # Define ticks color and width
        self.expSeries.frameTicks.setYRange((0, 1))

        s = self.expSeries.ui.splitter
        s.handle(1).setEnabled(True) # Allow to change splitter height
        s.setStyleSheet("background: 5px white;") # Define splitter background color 
        s.setHandleWidth(5) # Define splitter width



# -*- coding: utf-8 -*-
"""
Created on Fri Nov 24 11:11:27 2023

@author: clanglois1
"""

import os
from os.path import abspath

from inspect import getsourcefile

import numpy as np

from pyqtgraph.Qt import QtGui, QtCore
import pyqtgraph as pg
from PyQt5 import QtWidgets

from PyQt5.QtWidgets import QApplication

from inichord import general_functions as gf

if hasattr(QtCore.Qt, 'AA_EnableHighDpiScaling'):
    QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
    # enable highdpi scaling

if hasattr(QtCore.Qt, 'AA_UseHighDpiPixmaps'):
    QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)
    # use highdpi icons

path2thisFile = abspath(getsourcefile(lambda:0))
uiclass, baseclass = pg.Qt.loadUiType(os.path.dirname(path2thisFile) + "/NLMD_v2_TSG.ui")

class MainWindow(uiclass, baseclass):
    def __init__(self, parent):
        super().__init__()
        self.setupUi(self)
        self.parent = parent
        
        self.setWindowIcon(QtGui.QIcon('icons/filter_icon.png'))    
        
        self.expStack = parent.Current_stack

        self.denoised_Stack = np.copy(parent.Current_stack)

        self.x = 0
        self.y = 0
        
        self.patch_size = 5
        self.patch_distance = 6
        self.param_h = self.slider_h.value() / 10_0.0
        
        self.label_distance.setText("Patch Distance: " + str(self.patch_distance))
        self.label_size.setText("Patch Size: " + str(self.patch_size))
        self.label_h.setText("Parameter h: " + str(self.param_h))
        
        if self.denoised.isChecked():
            self.denoised.toggle()
            
        self.denoised.setCheckable(False)

        self.crosshair_v1= pg.InfiniteLine(angle=90, movable=False, pen=self.parent.color5)
        self.crosshair_h1 = pg.InfiniteLine(angle=0, movable=False, pen=self.parent.color5)
        
        self.plotIt = self.profiles.getPlotItem()
        self.plotIt.addLine(x = self.expSeries.currentIndex)
        
        self.proxy1 = pg.SignalProxy(self.expSeries.scene.sigMouseMoved, rateLimit=60, slot=self.mouseMoved)
        self.proxy4 = pg.SignalProxy(self.expSeries.ui.graphicsView.scene().sigMouseClicked, rateLimit=60, slot=self.mouseClick)
        
        self.slider_size.valueChanged.connect(self.size_changed)
        self.slider_distance.valueChanged.connect(self.distance_changed)
        self.slider_h.valueChanged.connect(self.h_changed)
        
        self.expSeries.timeLine.sigPositionChanged.connect(self.drawCHORDprofiles)
        self.preview.clicked.connect(self.denoiseStack)
        self.denoised.stateChanged.connect(self.drawCHORDprofiles)
        
        self.Validate_button.clicked.connect(self.validate)
        
        self.img_number = len(self.expStack)
        self.prgbar = 0 # Outil pour la bar de progression
        self.progressBar.setValue(self.prgbar)
        self.progressBar.setRange(0, self.img_number-1)
        
        self.type = self.expStack.dtype

        self.expStack = self.check_type(self.expStack) # Convert data to float32 if needed
        self.denoised_Stack = self.check_type(self.denoised_Stack) # Convert data to float32 if needed
            
        self.displayExpStack(self.expStack)
        self.defaultdrawCHORDprofiles()
        
        app = QApplication.instance()
        screen = app.screenAt(self.pos())
        geometry = screen.availableGeometry()
        
        self.move(int(geometry.width() * 0.1), int(geometry.height() * 0.15))
        self.resize(int(geometry.width() * 0.7), int(geometry.height() * 0.6))
        self.screen = screen
        
        self.Validate_button.setEnabled(False)
        self.mouseLock.setVisible(False)

    def check_type(self,data): # Check if the data has type uint8 or uint16 and modify it to float32
        datatype = data.dtype

        if datatype == "float64":
            data = gf.convertToUint8(data)
            self.data = data.astype(np.float32)
        if datatype == "uint16":
            data = gf.convertToUint8(data)
            self.data = data.astype(np.float32)
        elif datatype == "uint8":
            self.data = data.astype(np.float32)
        elif datatype == "float32":
            self.data = data
            
        return self.data

    def size_changed(self):
        value = self.slider_size.value()
        self.patch_size = value
        self.label_size.setText("Patch Size: " + str(value))
        self.denoiseSlice()
    
    def distance_changed(self):
        value = self.slider_distance.value()
        self.threshold = value
        self.label_distance.setText("Patch Distance: " + str(value))
        self.denoiseSlice()
        
    def h_changed(self):
        value = self.slider_h.value()
        self.param_h = value / 10_0.0        
        self.label_h.setText("Parameter h: " + str(self.param_h))
        self.denoiseSlice()

    def denoiseSlice(self):
        a = gf.NonLocalMeanDenoising(self.expStack[0, :, :], self.param_h, True, self.patch_size, self.patch_distance)
        
        self.denoised_Stack[0, :, :] = a

        self.displayExpStack(self.denoised_Stack)
        
        self.denoised.setEnabled(False)

    def denoiseStack(self):
        self.denoised.setEnabled(False)
        self.preview.setEnabled(False)
        
        for i in range(0,len(self.expStack[:, 0, 0])): # Apply parameter on each slice
            a = gf.NonLocalMeanDenoising(self.expStack[i, :, :], self.param_h, True, self.patch_size, self.patch_distance)
            self.denoised_Stack[i, :, :] = a

            QApplication.processEvents()    
            self.ValSlice = i
            self.progression_bar()
                
        self.denoised.setCheckable(True)
        self.denoised.setEnabled(True)  
        self.denoised.setChecked(True)
        self.preview.setEnabled(True)
        
        self.displayExpStack(self.denoised_Stack)
        self.Validate_button.setEnabled(True)
        
    def validate(self):
        self.parent.Current_stack = np.copy(self.denoised_Stack)
        self.parent.StackList.append(self.denoised_Stack)
        
        Combo_text = '\u2022 Manual NLMD denoising. Parameter h : ' + str(self.param_h) + '. Patch size : ' + str(self.patch_size) + '. Patch distance : ' + str(self.patch_distance)  
        Combo_data = self.denoised_Stack
        self.parent.choiceBox.addItem(Combo_text, Combo_data)

        self.parent.displayExpStack(self.parent.Current_stack)
        
        self.drawCHORDprofiles()  
        
        self.parent.Info_box.ensureCursorVisible()
        self.parent.Info_box.insertPlainText("\n \u2022 NLMD denoising is achieved.")     
        
        self.close()

    def defaultdrawCHORDprofiles(self):
        self.profiles.clear()
        self.profiles.setBackground(self.parent.color2)
        
        self.profiles.getPlotItem().hideAxis('bottom')
        self.profiles.getPlotItem().hideAxis('left')
     
    def drawCHORDprofiles(self):
        try:
            self.profiles.clear()
            line = self.plotIt.addLine(x = self.expSeries.currentIndex)
            line.setPen({'color': (42, 42, 42, 100), 'width': 2})
            
            self.legend = self.profiles.addLegend(horSpacing = 30, labelTextSize = '10pt', colCount = 1, labelTextColor = 'black', brush = self.parent.color6, pen = pg.mkPen(color=(0, 0, 0), width=1))
            
            pen = pg.mkPen(color=self.parent.color4, width=5) # Color and line width of the profile
            self.profiles.plot(self.expStack[:, self.x, self.y], pen=pen, name='Undenoised') # Plot of the profile
            
            styles = {"color": "black", "font-size": "40px", "font-family": "Noto Sans Cond"} # Style for labels
            self.profiles.setLabel("left", "Grayscale value", **styles) # Import style for Y label
            self.profiles.setLabel("bottom", "Slice", **styles) # Import style for X label
            
            font=QtGui.QFont('Noto Sans Cond', 9)
            
            self.profiles.getAxis("left").setTickFont(font) # Apply size of the ticks label
            self.profiles.getAxis("left").setStyle(tickTextOffset = 20) # Apply a slight offset
            self.profiles.getAxis("bottom").setTickFont(font) # Apply size of the ticks label
            self.profiles.getAxis("bottom").setStyle(tickTextOffset = 20) # Apply a slight offset
            
            self.profiles.getAxis('left').setTextPen('k') # Set the axis in black
            self.profiles.getAxis('bottom').setTextPen('k') # Set the axis in black
            
            self.profiles.setBackground(self.parent.color2)
            self.profiles.showGrid(x=True, y=True)
            
            if self.denoised.isChecked():
                pen2 = pg.mkPen(color=self.parent.color5, width=5) # Color and line width of the profile

                self.profiles.plot(self.denoised_Stack[:, self.x, self.y], pen=pen2, name='Denoised')
        except:
            pass

    def progression_bar(self): # Fonction relative à la barre de progression
        self.prgbar = self.ValSlice
        self.progressBar.setValue(self.prgbar)

    def displayExpStack(self, Series):
        self.expSeries.ui.histogram.hide()
        self.expSeries.ui.roiBtn.hide()
        self.expSeries.ui.menuBtn.hide()
        
        self.expSeries.addItem(self.crosshair_v1, ignoreBounds=True)
        self.expSeries.addItem(self.crosshair_h1, ignoreBounds=True) 
        
        view = self.expSeries.getView()
        state = view.getState()        
        self.expSeries.setImage(Series) 
        view.setState(state)
        
        view.setBackgroundColor(self.parent.color1)
        ROIplot = self.expSeries.getRoiPlot()
        ROIplot.setBackground(self.parent.color1)
        
        font=QtGui.QFont('Noto Sans Cond', 9)
        ROIplot.getAxis("bottom").setTextPen('k') # Apply size of the ticks label
        ROIplot.getAxis("bottom").setTickFont(font)
        
        self.expSeries.timeLine.setPen(color=self.parent.color3, width=15)
        self.expSeries.frameTicks.setPen(color=self.parent.color1, width=5)
        self.expSeries.frameTicks.setYRange((0, 1))

        s = self.expSeries.ui.splitter
        s.handle(1).setEnabled(True)
        s.setStyleSheet("background: 5px white;")
        s.setHandleWidth(5) 
        
    def mouseMoved(self, e):
        pos = e[0]

        if not self.mouseLock.isChecked():
            if self.expSeries.view.sceneBoundingRect().contains(pos):
    
                item = self.expSeries.view
                mousePoint = item.mapSceneToView(pos) 
                     
                self.crosshair_v1.setPos(mousePoint.x())
                self.crosshair_h1.setPos(mousePoint.y())

            self.x = int(mousePoint.x())
            self.y = int(mousePoint.y())
            
            if self.x >= 0 and self.y >= 0 and self.x < len(self.expStack[0, :, 0]) and self.y < len(self.expStack[0, 0, :]):
                self.drawCHORDprofiles()

    def mouseClick(self, e):
        pos = e[0]
        
        self.mouseLock.toggle()
        
        fromPosX = pos.scenePos()[0]
        fromPosY = pos.scenePos()[1]
        
        posQpoint = QtCore.QPointF()
        posQpoint.setX(fromPosX)
        posQpoint.setY(fromPosY)

        if self.expSeries.view.sceneBoundingRect().contains(posQpoint):
                
            item = self.expSeries.view
            mousePoint = item.mapSceneToView(posQpoint) 

            self.crosshair_v1.setPos(mousePoint.x())
            self.crosshair_h1.setPos(mousePoint.y())
                 
            self.x = int(mousePoint.x())
            self.y = int(mousePoint.y())
            
        if self.x >= 0 and self.y >= 0 and self.x < len(self.expStack[0, :, 0])and self.y < len(self.expStack[0, 0, :]):
            self.drawCHORDprofiles()


# -*- coding: utf-8 -*-

import os

from inspect import getsourcefile
from os.path import abspath

import numpy as np
import pyqtgraph as pg
import tifffile as tf
import time

from PyQt5.QtWidgets import QApplication
from PyQt5 import QtCore, QtGui
from PyQt5.QtWidgets import QMessageBox
from PyQt5.QtGui import QPixmap

from inichord import general_functions as gf

from skimage import morphology, filters, exposure
from skimage.measure import label, regionprops
from skimage.segmentation import expand_labels
from scipy import ndimage as ndi

path2thisFile = abspath(getsourcefile(lambda:0))
uiclass, baseclass = pg.Qt.loadUiType(os.path.dirname(path2thisFile) + "/GrainTreatment_v6_TSG.ui")

class MainWindow(uiclass, baseclass):
    def __init__(self, parent):
        super().__init__()
        
        self.setupUi(self)
        self.parent = parent
        
        self.setWindowIcon(QtGui.QIcon('icons/Grain_Icons.png'))
                
        self.flag_info = False # To display Otsu n°1 value or not
        self.flag_info_labels = False # To display grain labels value or not
        self.flag_info_labels_metric = False # No metric at the beginning
        self.flag_PixelSize = False # No consideration of pixel size at the opening
        self.flag_info_overlay = False # No consideration of the overlay map at the opening
        self.flag_info_filtered = False # No consideration of excluded pixels (pxls) at the opening
        self.flag_info_filtered2 = False # No consideration of excluded pixels (µm) at the opening
        self.flag_Border = False #No consideration of excluded borders at the opening
        
        self.x = 0
        self.y = 0

        self.crosshair_v1= pg.InfiniteLine(angle=90, movable=False, pen=self.parent.color5)
        self.crosshair_h1 = pg.InfiniteLine(angle=0, movable=False, pen=self.parent.color5)
        
        self.crosshair_v2= pg.InfiniteLine(angle=90, movable=False, pen=self.parent.color5)
        self.crosshair_h2 = pg.InfiniteLine(angle=0, movable=False, pen=self.parent.color5)
        
        self.crosshair_v3= pg.InfiniteLine(angle=90, movable=False, pen=self.parent.color5)
        self.crosshair_h3 = pg.InfiniteLine(angle=0, movable=False, pen=self.parent.color5)
        
        self.crosshair_v4= pg.InfiniteLine(angle=90, movable=False, pen=self.parent.color5)
        self.crosshair_h4 = pg.InfiniteLine(angle=0, movable=False, pen=self.parent.color5)
        
        self.crosshair_v5= pg.InfiniteLine(angle=90, movable=False, pen=self.parent.color5)
        self.crosshair_h5 = pg.InfiniteLine(angle=0, movable=False, pen=self.parent.color5)
        
        self.proxy1 = pg.SignalProxy(self.KADSeries.scene.sigMouseMoved, rateLimit=60, slot=self.mouseMoved)
        self.proxy2 = pg.SignalProxy(self.KADSeries.ui.graphicsView.scene().sigMouseClicked, rateLimit=60, slot=self.mouseClick)
        
        self.proxy3 = pg.SignalProxy(self.FiltKADSeries.scene.sigMouseMoved, rateLimit=60, slot=self.mouseMoved)
        self.proxy4 = pg.SignalProxy(self.FiltKADSeries.ui.graphicsView.scene().sigMouseClicked, rateLimit=60, slot=self.mouseClick)

        self.proxy5 = pg.SignalProxy(self.Otsu1Series.scene.sigMouseMoved, rateLimit=60, slot=self.mouseMoved)
        self.proxy6 = pg.SignalProxy(self.Otsu1Series.ui.graphicsView.scene().sigMouseClicked, rateLimit=60, slot=self.mouseClick)

        self.proxy7 = pg.SignalProxy(self.Binary1Series.scene.sigMouseMoved, rateLimit=60, slot=self.mouseMoved)
        self.proxy8 = pg.SignalProxy(self.Binary1Series.ui.graphicsView.scene().sigMouseClicked, rateLimit=60, slot=self.mouseClick)

        self.proxy9 = pg.SignalProxy(self.LabelsSeries.scene.sigMouseMoved, rateLimit=60, slot=self.mouseMoved)
        self.proxy10 = pg.SignalProxy(self.LabelsSeries.ui.graphicsView.scene().sigMouseClicked, rateLimit=60, slot=self.mouseClick)

        self.OpenData.clicked.connect(self.loaddata) # Load a data
        self.ComputeClass_bttn.clicked.connect(self.Otsu1) # Computation of the Otsu (classes creation)
        self.Threshold_bttn.clicked.connect(self.Binary_1) # Computation of the Otsu thresholding
        self.ComputeLabels_bttn.clicked.connect(self.Grain_labeling) # Labeling of grains
        self.Save_bttn.clicked.connect(self.Save_results) # Saving process (processing steps, results, infos)
        self.Push_validate.clicked.connect(self.validate_data)
        self.Grainsize_bttn.clicked.connect(self.labels_computation)
        self.exclude_border_bttn.clicked.connect(self.exclude_border) # Exclude labels with borders
                
        self.PixelSize_edit.setText("Add a pixel size (µm).")
        self.PixelSize_edit.editingFinished.connect(self.changeText) # Take into account the pixel size
        
        self.PresetBox.currentTextChanged.connect(self.auto_set) # Allow different pre-set to be used for computation
        self.spinBox_filter.valueChanged.connect(self.Filter_changed) # Change initial filtering
        self.Den_spinbox.valueChanged.connect(self.denoise_labels) # Change denoising value of the labels
        self.Filter_labelBox.valueChanged.connect(self.Filter_labels) # To exclude small grains
        self.ChoiceBox.currentTextChanged.connect(self.ViewLabeling) # Change displayed map
        
        self.Grainsize_bttn.setEnabled(False)
        
        self.label_filterdiameter.setText("Exclude \u2300 < x(pxls)")
        
        self.defaultIV() # Hide the PlotWidget until a data has been loaded
        self.mouseLock.setVisible(False)
        
        # Icons sizes management for QMessageBox
        self.pixmap = QPixmap("icons/Grain_Icons.png")
        self.pixmap = self.pixmap.scaled(100, 100)
             
        try:
            if hasattr(parent, 'KAD') : # Choice of KAD if only available
                self.InitKAD_map = parent.KAD
                self.flag_KAD = True
                
            if hasattr(parent, 'contour_map') : # Choice of contour map if only avalaible
                self.InitKAD_map = parent.contour_map
                self.flag_KAD = False
                
            if hasattr(parent, 'KAD') and hasattr(parent, 'contour_map'):
                self.show_choice_message() # Choice of the map if the two are available
                
            self.StackDir = self.parent.StackDir
            self.run_init_computation()
        except:
            pass
            
        app = QApplication.instance()
        screen = app.screenAt(self.pos())
        geometry = screen.availableGeometry()
        
        # Control window position and dimensions
        self.move(int(geometry.width() * 0.05), int(geometry.height() * 0.05))
        self.resize(int(geometry.width() * 0.9), int(geometry.height() * 0.6))
        self.screen = screen
        
#%% Functions
    def show_choice_message(self): # Qmessage box for the try import at the initialization 
        msg_box = QMessageBox()
        msg_box.setWindowTitle("Choice of the data")
        msg_box.setText("Data to use :")
    
        btn_kad = msg_box.addButton("KAD map", QMessageBox.ActionRole)
        btn_contour = msg_box.addButton("Contour map", QMessageBox.ActionRole)

        msg_box.exec_()
    
        if msg_box.clickedButton() == btn_kad:
            self.InitKAD_map = self.parent.KAD
            self.flag_KAD = True
        elif msg_box.clickedButton() == btn_contour:
            self.InitKAD_map = self.parent.contour_map
            self.flag_KAD = False

    def data_choice(self): # Allow to apply other treatment depending if the data is a KAD one
        msg = QMessageBox.question(self, 'Restored grains', 'Is it a KAD data ?')
        if msg == QMessageBox.Yes:
            self.flag_KAD = True
        if msg == QMessageBox.No:
            self.flag_KAD = False

    def loaddata(self): # Opening of a 2D array (KAD array)
        self.defaultIV() # Hide the PlotWidget until a data has been loaded
        self.ChoiceBox.clear() 
        
        self.StackLoc, self.StackDir = gf.getFilePathDialog("Open map (*.tiff)") 
        
        checkimage = tf.TiffFile(self.StackLoc[0]).asarray() # Check for dimension. If 2 dimensions : 2D array. If 3 dimensions : stack of images
        if checkimage.ndim != 2: # Check if the data is a KAD map (2D array)
            self.parent.popup_message("Grain segmentation","Please import a 2D array",'icons/Grain_Icons.png')
            return
        
        self.data_choice()
        
        self.InitKAD_map = tf.TiffFile(self.StackLoc[0]).asarray()
        self.InitKAD_map = np.flip(self.InitKAD_map, 0)
        self.InitKAD_map = np.rot90(self.InitKAD_map, k=1, axes=(1, 0))
        
        self.InitKAD_map = np.nan_to_num(self.InitKAD_map) # Exclude NaN value if needed
        self.InitKAD_map = (self.InitKAD_map - np.min(self.InitKAD_map)) / (np.max(self.InitKAD_map) - np.min(self.InitKAD_map)) # Normalization step
        self.InitKAD_map = exposure.equalize_adapthist(self.InitKAD_map, kernel_size=None, clip_limit=0.01, nbins=256) # CLAHE step
        
        self.displayExpKAD(self.InitKAD_map) # Display of the map
        
        # If data is too large, hence the labeling step is not performed to avoid long computation prior checking if the data is good to be labeled
        if (len(self.InitKAD_map) * len(self.InitKAD_map[0])) < 1_000_000 :
            self.auto_set()
        else:
            self.high_auto_set()

    def run_init_computation(self): # Run a first analysis automatically
        self.InitKAD_map = np.nan_to_num(self.InitKAD_map) # Exclude NaN value if needed
        self.InitKAD_map = (self.InitKAD_map - np.min(self.InitKAD_map)) / (np.max(self.InitKAD_map) - np.min(self.InitKAD_map)) # Normalization step
        self.InitKAD_map = exposure.equalize_adapthist(self.InitKAD_map, kernel_size=None, clip_limit=0.01, nbins=256) # CLAHE step
        
        self.displayExpKAD(self.InitKAD_map) # Display of the map
        
        # If data is too large, hence the labeling step is not performed to avoid long computation prior checking if the data is good to be labeled
        if (len(self.InitKAD_map) * len(self.InitKAD_map[0])) < 1_000_000 :
            self.auto_set()
        else:
            self.high_auto_set()

    def auto_set(self): # Allow different pre-set to be used
        self.Preset_choice = self.PresetBox.currentText()

        if self.Preset_choice == "Undeformed sample":
            self.spinBox_filter.setValue(0.005)
            self.ClassBox.setValue(4)
            self.ThresholdBox.setValue(1)

            self.Filter_changed() # Compute the filtered map
            self.Otsu1() # Compute a first Otsu map
            self.Binary_1() # Compute a first binary map
            self.Grain_labeling() # Compute a labeled map (undenoised)      
            
        elif self.Preset_choice == "Slighlty deformed sample":
            self.spinBox_filter.setValue(0.02)
            self.ClassBox.setValue(5)
            self.ThresholdBox.setValue(2)
            
            self.Filter_changed() # Compute the filtered map
            self.Otsu1() # Compute a first Otsu map
            self.Binary_1() # Compute a first binary map
            self.Grain_labeling() # Compute a labeled map (undenoised)
            
        elif self.Preset_choice == "Heavily deformed sample":
            self.spinBox_filter.setValue(0.025)
            self.ClassBox.setValue(6)
            self.ThresholdBox.setValue(3)
            
            self.Filter_changed() # Compute the filtered map
            self.Otsu1() # Compute a first Otsu map
            self.Binary_1() # Compute a first binary map
            self.Grain_labeling() # Compute a labeled map (undenoised)

    def high_auto_set(self):
            msg = QMessageBox()
            msg.setIconPixmap(self.pixmap)
            msg.setWindowTitle("Grain boundaries determination")
            msg.setText("Labeling step is not apply to limit calculation times (large data map).")
            msg.setWindowIcon(QtGui.QIcon('icons/Grain_Icons.png'))
            msg.exec_()
            
            self.spinBox_filter.setValue(0.005)
            self.ClassBox.setValue(3)
            self.ThresholdBox.setValue(1)

            self.Filter_changed() # Compute the filtered map
            self.Otsu1() # Compute a first Otsu map
            self.Binary_1() # Compute a first binary map
            
    def changeText(self): # Importation of pixel size value after writing in the GUI
        self.pixelSize = float(self.PixelSize_edit.text())
        self.flag_PixelSize = True # Consideration of pixel size

    def Filter_changed(self): # KAD filtering processes
        self.Filter_choice = self.FilterBox.currentText()
        
        if self.Filter_choice == "Butterworth (HP) filter":
            self.spinBox_filter.setRange(0.005,0.5)
            self.spinBox_filter.setSingleStep(0.005)
            
            self.FilteredKAD_map = np.copy(self.InitKAD_map)
            self.Filter_value = self.spinBox_filter.value()
            self.FilteredKAD_map = filters.butterworth(self.FilteredKAD_map,self.Filter_value,True,8)
            
        elif self.Filter_choice == "Mean filter":
            self.spinBox_filter.setRange(0,10)
            self.spinBox_filter.setSingleStep(1)
            
            self.FilteredKAD_map = np.copy(self.InitKAD_map)
            self.Filter_value = self.spinBox_filter.value()
            self.FilteredKAD_map = filters.gaussian(self.FilteredKAD_map, self.Filter_value)
            
        elif self.Filter_choice == "Top-hat":
            self.spinBox_filter.setRange(1,20)
            self.spinBox_filter.setSingleStep(1)
            self.FilteredKAD_map = np.copy(self.InitKAD_map)
            self.Filter_value = self.spinBox_filter.value()
            footprint = morphology.disk(self.Filter_value)
            self.FilteredKAD_map = morphology.white_tophat(self.FilteredKAD_map, footprint)

        self.displayFilteredKAD(self.FilteredKAD_map) # Display the map after load

    def Otsu1(self): # Segment map into classes
        self.Otsu1_Value = self.ClassBox.value()

        # Segmentation of the intensities for a given number of classes
        thresholds = filters.threshold_multiotsu(self.FilteredKAD_map, classes = self.Otsu1_Value) # Definition of the threshold values
        self.regions = np.digitize(self.FilteredKAD_map, bins=thresholds) # Using the threshold values, we generate the regions.

        self.flag_info = True 
        self.displayOtsu1(self.regions) # Display the Otsu map

    def Binary_1(self): # Binarization of the Otsu map for a given threshold level
        self.Binary1_Value = self.ThresholdBox.value()
        
        self.regions2 = np.copy(self.regions)

        var_up = np.where(self.regions >= self.Binary1_Value) # Search for every value higher or equal to threshold
        var_down = np.where(self.regions < self.Binary1_Value) # Search for every value below the threshold
        self.regions2[var_up] = 1 # Replace values by 1 ==> Binary image created
        self.regions2[var_down] = 0 # Replace values by 1 ==> Binary image created

        self.regions3 = ndi.binary_closing(self.regions2) # Closing step 
        self.binary_regions = 1-(ndi.binary_dilation(self.regions3, iterations = 1)) # Dilation to increase connectivity

        self.displayBinary1(self.binary_regions) # Display the thresholded map

    def Grain_labeling(self):
        self.label_img = label(self.binary_regions, connectivity=2) # Labeling of the thresholded map
        self.d = np.zeros(np.amax(self.label_img) + 1) # Array of 0 value to store the area of grains
    
        self.flag_DenLabels = False
        self.labels_computation()
        
        self.Grainsize_bttn.setEnabled(True)
        
    def denoise_labels(self): # Fill labels with holes inside
        # Expansion of labels ==> Get rid of grains boundaries and dust or holes
        distance_val = self.Den_spinbox.value()
        
        self.flag_DenLabels = True
        self.label_img_refined = expand_labels(self.label_img, distance=distance_val)
        
        self.displaylabels(self.label_img_refined) # Display the labeled image

    def labels_computation(self):
        # Computation of equivalent diameters
        if self.flag_DenLabels == False:
            self.Var_Labels = np.copy(self.label_img)
        elif self.flag_DenLabels == True:
            self.Var_Labels = np.copy(self.label_img_refined)
            
        self.img_diameter = np.zeros(self.label_img.shape) #Array of labeled grains with associated equivalent diameters
        
        self.img_area = np.zeros(self.label_img.shape) #Array of labeled grains with associated area (pxls)
        self.form_factor = np.zeros(np.amax(self.label_img) + 1) # Form factor
        self.img_formfactor = np.zeros(self.label_img.shape) # Form factor map
        
        for i,region in enumerate(regionprops(self.Var_Labels)):
            
            i=i+1
            self.d[i] = region.area_filled
            
            major_axis_length = region.major_axis_length
            minor_axis_length = region.minor_axis_length
            if minor_axis_length == 0:
                self.form_factor[i] = 0
            else:
                self.form_factor[i] = major_axis_length / minor_axis_length
                
            var = np.where(self.Var_Labels == i)
            self.img_diameter[var] = self.d[i]
            self.img_area[var] = self.d[i]
            self.img_formfactor[var] = self.form_factor[i]      
            
        var = np.where(self.img_diameter == 0)
        
        self.img_diameter = ((2*np.sqrt(self.img_diameter/np.pi)) + 2)
        self.img_diameter[var] = 0
        
        # Correction of labels and diameter values
        var = np.where(self.img_diameter == 0)

        x = 1
        y = 2

        self.Corrected_img_diameter = np.copy(self.img_diameter) # Map of the grains (diameter value map) with border corrected
        self.Corrected_label_img = np.copy(self.Var_Labels) # Map of the grains (labeled value map) with border corrected

        for i in range(len(var[0])):
            varx = var[0][i] # X position of the pixel
            vary = var[1][i] # X position of the pixel
                
            var_diameter = self.img_diameter[varx-x:varx+y,vary-x:vary+y]
            var_label = self.Var_Labels[varx-x:varx+y,vary-x:vary+y]
                
            if var_diameter.size != 0:
          
                value_diameter = np.max(var_diameter)
                value_label = np.max(var_label)
                
                self.Corrected_img_diameter[varx,vary] = value_diameter
                self.Corrected_label_img[varx,vary] = value_label

        # Creation of the overlay map (KAD and grain boundaries)
        self.overlay_KAD_GB = np.copy(self.InitKAD_map)
        self.overlay_KAD_GB[var] = 1   

        # Creation of items in the QComboBox
        self.ChoiceBox.clear() 

        Combo_text = 'Labeled grains'
        Combo_data = self.Corrected_label_img
        self.ChoiceBox.addItem(Combo_text, Combo_data)

        Combo_text = 'Grains diameter (pxls)'
        Combo_data = self.Corrected_img_diameter
        self.ChoiceBox.addItem(Combo_text, Combo_data)
        
        self.flag_info_labels = False
        self.displaylabels(self.Corrected_label_img) # Display the labeled image
        
        if self.flag_PixelSize == True:
            self.Corrected_img_diameter_metric = np.copy(self.Corrected_img_diameter)
            self.Corrected_img_diameter_metric = self.Corrected_img_diameter_metric * self.pixelSize
            
            Combo_text = 'Grains diameter (µm)'
            Combo_data = self.Corrected_img_diameter_metric
            self.ChoiceBox.addItem(Combo_text, Combo_data)
            
            self.img_area_metric = np.copy(self.img_area)
            self.img_area_metric = self.img_area_metric * self.pixelSize
            
        Combo_text = 'Overlay map-GB'
        Combo_data = self.overlay_KAD_GB
        self.ChoiceBox.addItem(Combo_text, Combo_data)
        
        self.Filter_labels() # After labels exclusion
        self.displaylabels(self.Corrected_label_img) # Display the labeled image
            
        # Finished message
        self.parent.popup_message("Grain boundaries determination","Properties computed.",'icons/Grain_Icons.png')
        
    def extract_value_list(self): # Extract informations       
        # Equivalent diameter data 
        self.extract_diameter = np.copy(self.d) # Area list
        self.extract_diameter = (2*np.sqrt(self.extract_diameter/np.pi)) + 2 # Conversion in diameter

        self.filtered_diameter = np.copy(self.extract_diameter) # Copy
        var = np.where(self.filtered_diameter <= self.Filter_labelValue) # Search for filtering
        self.filtered_diameter[var] = 0 # Replace value by 0
        
        # Area data 
        self.area_list_pxls = np.copy(self.d)
        self.Filtered_area_list_pxls = np.copy(self.area_list_pxls)
        self.Filtered_area_list_pxls[var] = 0
                
        # Form factor data
        self.form_factor_list = np.copy(self.form_factor)
        self.Filtered_form_factor_list = np.copy(self.form_factor_list)
        self.Filtered_form_factor_list[var] = 0
                
        if self.flag_PixelSize == True:
            self.extract_diameter_metric = self.extract_diameter * self.pixelSize
            self.filtered_diameter_metric = self.filtered_diameter * self.pixelSize
            
            # Area data
            self.area_list_metric = np.copy(self.d) * self.pixelSize
            self.Filtered_area_list_metric = np.copy(self.area_list_metric)
            self.Filtered_area_list_metric[var] = 0
            
        if self.flag_Border == True:
            self.area_list_pxls = np.delete(self.area_list_pxls, self.label_border)
            self.Filtered_area_list_pxls = np.delete(self.Filtered_area_list_pxls, self.label_border)
            self.Filtered_area_list_pxls = self.Filtered_area_list_pxls[self.Filtered_area_list_pxls != 0]

            self.extract_diameter = np.delete(self.extract_diameter, self.label_border)
            self.filtered_diameter = np.delete(self.filtered_diameter, self.label_border)
            self.filtered_diameter = self.filtered_diameter[self.filtered_diameter != 0]
            
            self.form_factor_list = np.delete(self.form_factor_list, self.label_border)
            self.Filtered_form_factor_list = np.delete(self.Filtered_form_factor_list, self.label_border)
            self.Filtered_form_factor_list = self.Filtered_form_factor_list[self.Filtered_form_factor_list != 0]
            
            if self.flag_PixelSize == True:
                self.area_list_metric = np.delete(self.area_list_metric, self.label_border)
                self.Filtered_area_list_metric = np.delete(self.Filtered_area_list_metric, self.label_border)
                self.Filtered_area_list_metric = self.Filtered_area_list_metric[self.Filtered_area_list_metric != 0]

                self.extract_diameter_metric = np.delete(self.extract_diameter_metric, self.label_border)
                self.filtered_diameter_metric = np.delete(self.filtered_diameter_metric, self.label_border)
                self.filtered_diameter_metric = self.filtered_diameter_metric[self.filtered_diameter_metric != 0]

        else:
            self.Filtered_area_list_pxls = self.Filtered_area_list_pxls[self.Filtered_area_list_pxls != 0]
            self.filtered_diameter = self.filtered_diameter[self.filtered_diameter != 0]
            self.Filtered_form_factor_list = self.Filtered_form_factor_list[self.Filtered_form_factor_list != 0]
            
            if self.flag_PixelSize == True:
                self.Filtered_area_list_metric = self.Filtered_area_list_metric[self.Filtered_area_list_metric != 0]
                self.filtered_diameter_metric = self.filtered_diameter_metric[self.filtered_diameter_metric != 0]

    def ViewLabeling(self):
        self.view_choice = self.ChoiceBox.currentText()
        
        self.flag_info_labels = False # To display grain labels value or not
        self.flag_info_labels_metric = False # No metric at the beginning
        self.flag_info_overlay = False # No consideration of the overlay map at the opening
        self.flag_info_filtered = False # No consideration of pixel size at the opening
        self.flag_info_filtered2 = False # No consideration of the overlay map at the opening
        
        if self.view_choice == "Labeled grains":
            self.displaylabels(self.Corrected_label_img) # Display the map after load
            self.flag_info_labels = False
            
        if self.view_choice == "Grains diameter (pxls)":
            self.displaylabels(self.Corrected_img_diameter) # Display the map after load
            self.flag_info_labels = True
            
        if self.view_choice == "Grains diameter (µm)":
            self.displaylabels(self.Corrected_img_diameter_metric) # Display the map after load
            self.flag_info_labels_metric = True
            
        if self.view_choice == "Overlay map-GB":
            self.displaylabels(self.overlay_KAD_GB) # Display the map after load
            self.flag_info_overlay = True
            
        if self.view_choice == "Grain diameter pxls (excluded \u2300)":
            self.displaylabels(self.filter_labeldiameter) # Display the map after load
            self.flag_info_filtered = True

        if self.view_choice == "Grain diameter µm (excluded \u2300)":
            self.displaylabels(self.filter_labeldiameter_metric) # Display the map after load
            self.flag_info_filtered2 = True

    def Filter_labels(self): # Labels filtering (== 0 if value <= the given excluding value)
        self.Filter_labelValue = self.Filter_labelBox.value()
        
        var = np.where(self.Corrected_img_diameter <= self.Filter_labelValue)
        
        # Equivalent diameter after exclusion of small labels
        self.filter_labeldiameter = np.copy(self.Corrected_img_diameter)
        self.filter_labeldiameter[var] = 0
                
        try : # Try to find if the data already exist to rewrite it
            index = self.ChoiceBox.findText('Grain diameter pxls (excluded \u2300)')
            self.ChoiceBox.removeItem(index)
            
            Combo_text = 'Grain diameter pxls (excluded \u2300)'
            Combo_data = self.filter_labeldiameter
            self.ChoiceBox.addItem(Combo_text, Combo_data)
        except:
            pass
        
        if self.flag_PixelSize == True: 
            self.filter_labeldiameter_metric = np.copy(self.Corrected_img_diameter_metric)
            self.filter_labeldiameter_metric[var] = 0
            
            try : # Try to find if the data already exist to rewrite it
                index = self.ChoiceBox.findText('Grain diameter µm (excluded \u2300)')
                self.ChoiceBox.removeItem(index)
                
                Combo_text = 'Grain diameter µm (excluded \u2300)'
                Combo_data = self.filter_labeldiameter_metric
                self.ChoiceBox.addItem(Combo_text, Combo_data)
            except:
                pass
            
        self.displaylabels(self.filter_labeldiameter) # Display the map after load

        # Area after exclusion of small labels in pxls
        self.filter_area_pxls = np.copy(self.img_area)
        self.filter_area_pxls[var] = 0
        
        if self.flag_PixelSize == True: 
            # Area after exclusion of small labels in µm
            self.filter_area_metric = np.copy(self.img_area_metric)
            self.filter_area_metric[var] = 0
        
        # Form factor after exclusion of small labels
        self.filter_form_factor = np.copy(self.img_formfactor)
        self.filter_form_factor[var] = 0

    def exclude_border(self): 
        self.flag_Border = True
        # Get position (pxls) of borders
        border_positions = self.get_border_positions(self.Var_Labels) 
        # Get labels value
        border_intensities = self.get_border_intensities(self.Var_Labels, border_positions)
        self.label_border = np.unique(border_intensities)
        # Replace labels with 0
        self.Var_Labels_cleaned = self.zero_out_matching_intensities(self.Var_Labels, border_intensities) 
        # Get all the 0 value positions 
        zero_positions = self.get_zero_positions(self.Var_Labels_cleaned)

        # Filter application on the other main maps ==> Pixel maps
        # Area map (pixels)
        self.img_area_cleaned = self.zero_out_positions(self.img_area, zero_positions)
        # Equivalent diameter (pixels)
        self.Corrected_img_diameter_cleaned = self.zero_out_positions(self.Corrected_img_diameter, zero_positions)
        # Form factor
        self.img_formfactor_cleaned = self.zero_out_positions(self.img_formfactor, zero_positions)
        
        # Filter application on the other main maps ==> metric maps
        if self.flag_PixelSize == True: 
            # Area map (µm)
            self.img_area_metric_cleaned = self.zero_out_positions(self.img_area_metric, zero_positions)
            # Equivalent diameter (µm)
            self.Corrected_img_diameter_metric_cleaned = self.zero_out_positions(self.Corrected_img_diameter_metric, zero_positions)
            
        # Filter application on the other main maps ==> pixel maps
        # Filtered area map (pixels)
        self.filter_area_pxls_cleaned = self.zero_out_positions(self.filter_area_pxls, zero_positions)
        # Filtered Equivalent diameter (pixels)
        self.filter_labeldiameter_cleaned = self.zero_out_positions(self.filter_labeldiameter, zero_positions)
        # Filtered form factor
        self.filter_form_factor_cleaned = self.zero_out_positions(self.filter_form_factor, zero_positions)
        
        # Filter application on the other main maps ==> metric maps
        if self.flag_PixelSize == True: 
            # Filtered area map (µm)
            self.filter_area_metric_cleaned = self.zero_out_positions(self.filter_area_metric, zero_positions)
            # Filtered Equivalent diameter (µm)
            self.filter_labeldiameter_metric_cleaned = self.zero_out_positions(self.filter_labeldiameter_metric, zero_positions)

        # Finished message
        self.parent.popup_message("Grain boundaries determination","Border excluded.",'icons/Grain_Icons.png')

    def get_border_positions(self,image):# Fonction pour obtenir les positions des pixels des bordures
        border_positions = []
        rows, cols = image.shape
        for i in range(rows):
            for j in range(cols):
                if i == 0 or i == rows - 1 or j == 0 or j == cols - 1:
                    border_positions.append((i, j))
        return border_positions

    def get_border_intensities(self,image, border_positions):# Fonction pour obtenir les intensités des pixels des bordures
        border_intensities = [image[i, j] for i, j in border_positions]
        return border_intensities
    
    def zero_out_matching_intensities(self,image, border_intensities):# Fonction pour mettre à zéro les pixels ayant la même intensité que les bordures
        image_cleaned = np.copy(image)
        for intensity in border_intensities:
            image_cleaned[image == intensity] = 0
        return image_cleaned

    def get_zero_positions(self,image):
        zero_positions = np.argwhere(image == 0)# Fonction pour obtenir les positions des pixels mis à zéro
        return zero_positions

    def zero_out_positions(self,image, positions):# Fonction pour mettre à zéro les pixels aux positions spécifiées
        image2 = np.copy(image)
        for i, j in positions:
            image2[i, j] = 0
            
        return image2

    def Compute_clustered_profiles(self):
        # Try to open the current_stack and check if the dim are the same than the labeled img
        # Else, ask to open the image series
        try : 
            serie = self.parent.Current_stack
        except:
            StackLoc, StackDir = gf.getFilePathDialog("Stack of images (*.tiff)")  # Ask to open the stack of images
            serie = tf.TiffFile(StackLoc[0]).asarray() # Import the array
            serie = np.flip(serie, 1)
            serie = np.rot90(serie, k=1, axes=(2, 1))
            
        # Convert labeled image as integer
        Labels_int = np.zeros((len(self.Corrected_label_img),len(self.Corrected_label_img[0])), dtype = int)

        for i in range(0,len(self.Corrected_label_img)):
            for j in range(0,len(self.Corrected_label_img[0])):
                Labels_int[i,j] = int(self.Corrected_label_img[i,j])

        # Expansion of labels ==> Get rid of grains boundaries !
        Labels_int = expand_labels(Labels_int, distance=10)

        # Definition of the mean profiles for each label
        moyen_profil=np.zeros((len(regionprops(Labels_int)),len(serie[:,0,0])))
        
        try :
            for i in range (len(serie[:,0,0])) :
                regions = regionprops(Labels_int, intensity_image=serie[i,:,:])
                for j in range (len(regions)) :
                    moyen_profil[j][i]=regions[j].mean_intensity 
        except:
            self.parent.popup_message("Grain boundaries determination","Computation of clustered profiles failed. Check for data.",'icons/Grain_Icons.png')
            return
        
        # Creation of the clustered profiles list
        liste_clusters = np.copy(moyen_profil)

        self.liste = []

        for i in range (0,np.max(Labels_int)):
            var = liste_clusters[i,:]
            self.liste.append(var)

        self.liste = np.dstack(self.liste)
        self.liste = np.swapaxes(self.liste, 0, 1)
        
        self.Labels_int = Labels_int

    def Save_results(self):
        ti = time.strftime("%Y-%m-%d__%Hh-%Mm-%Ss") # Absolute time 
        
        directory = "Grain_segmentation_" + ti # Name of the main folder
        PathDir = os.path.join(self.StackDir, directory)  # where to create the main folder
        os.mkdir(PathDir)  # Create main folder
        
        # Information (.TXT) step
        with open(PathDir + '\Grain boundaries determination.txt', 'w') as file:
            file.write("KAD data: " + str(self.flag_KAD))
            file.write("\nFiltering parameter: " + str(self.Filter_choice) + " - " + (str(self.Filter_value)))   
            file.write("\nOtsu class: "+ str(self.Otsu1_Value) + "\nThresholded classes (keep values equal or higher than): " + str(self.Binary1_Value))   
            file.write("\nLabel denoising step: "+ str(self.flag_DenLabels))
            file.write("\nLabel denoising value: "+ str(self.Den_spinbox.value()))
            file.write("\nGrain diameter excluded (below): "+ str(self.Filter_labelValue))
            
            if self.flag_PixelSize == True:
                file.write("\nPixel size (µm): " + str(self.pixelSize))
        
        # Creation of the processing folder + save
        processing_folder = "Processing_step" # Name of the sub-folder
        SubPathDir = os.path.join(PathDir, processing_folder) # Sub-folder for processing step
        os.mkdir(SubPathDir)  # Create sub-folder
        
        tf.imwrite(SubPathDir + '/Map_CLAHE.tiff', np.rot90(np.flip(self.InitKAD_map, 0), k=1, axes=(1, 0)))
        tf.imwrite(SubPathDir + '/Filtered_map.tiff', np.rot90(np.flip(self.FilteredKAD_map, 0), k=1, axes=(1, 0)))  
        tf.imwrite(SubPathDir + '/Otsu.tiff', np.rot90(np.flip(self.regions, 0), k=1, axes=(1, 0)).astype('float32')) 
        tf.imwrite(SubPathDir + '/Binary_Otsu.tiff', np.rot90(np.flip(self.binary_regions, 0), k=1, axes=(1, 0)))

        if self.flag_Border == True:
            self.Corrected_label_img = self.Var_Labels_cleaned
            self.img_area = self.img_area_cleaned
            self.Corrected_img_diameter = self.Corrected_img_diameter_cleaned
            self.img_formfactor = self.img_formfactor_cleaned
            
            self.filter_area_pxls = self.filter_area_pxls_cleaned
            self.filter_labeldiameter = self.filter_labeldiameter_cleaned
            self.filter_form_factor = self.filter_form_factor_cleaned
            
            if self.flag_PixelSize == True:
                self.img_area_metric = self.img_area_metric_cleaned
                self.Corrected_img_diameter_metric = self.Corrected_img_diameter_metric_cleaned
                self.filter_area_metric = self.filter_area_metric_cleaned
                self.filter_labeldiameter_metric = self.filter_labeldiameter_metric_cleaned

        # Creation of the pixel data folder  + save
        Pxls_folder = "Pixel data" # Name of the sub-folder for filtered data 
        Pxls_SubPathDir = os.path.join(PathDir, Pxls_folder) # Sub-folder for filtered data
        os.mkdir(Pxls_SubPathDir)  # Create sub-folder
        
        tf.imwrite(Pxls_SubPathDir + '/Labeled_img.tiff', np.rot90(np.flip(self.Corrected_label_img, 0), k=1, axes=(1, 0)).astype('float32')) 
        tf.imwrite(Pxls_SubPathDir + '/Area_pxls.tiff', np.rot90(np.flip(self.img_area, 0), k=1, axes=(1, 0))) # Area map in pxls
        tf.imwrite(Pxls_SubPathDir + '/Equivalent_diameter_pxls.tiff', np.rot90(np.flip(self.Corrected_img_diameter, 0), k=1, axes=(1, 0)))
        tf.imwrite(Pxls_SubPathDir + '/form_factor_map.tiff', np.rot90(np.flip(self.img_formfactor, 0), k=1, axes=(1, 0))) # Area map in pxls

        tf.imwrite(Pxls_SubPathDir + '/Filtered_area_pxls.tiff', np.rot90(np.flip(self.filter_area_pxls, 0), k=1, axes=(1, 0))) # Area map in pxls
        tf.imwrite(Pxls_SubPathDir + '/Filtered_equivalent_diameter_pxls.tiff', np.rot90(np.flip(self.filter_labeldiameter, 0), k=1, axes=(1, 0)))
        tf.imwrite(Pxls_SubPathDir + '/Filtered_form_factor_map.tiff', np.rot90(np.flip(self.filter_form_factor, 0), k=1, axes=(1, 0))) # Area map in pxls

        # Creation of the metric data folder 
        if self.flag_PixelSize == True:
            Metric_folder = "Metric data" # Name of the sub-folder for metric data 
            Metric_SubPathDir = os.path.join(PathDir, Metric_folder) # Sub-folder for metric data
            os.mkdir(Metric_SubPathDir)  # Create sub-folder
        
            tf.imwrite(Metric_SubPathDir + '/Area_µm.tiff', np.rot90(np.flip(self.img_area_metric, 0), k=1, axes=(1, 0))) # Area map in µm
            tf.imwrite(Metric_SubPathDir + '/Equivalent_diameter_µm.tiff', np.rot90(np.flip(self.Corrected_img_diameter_metric, 0), k=1, axes=(1, 0)))
            
            tf.imwrite(Metric_SubPathDir + '/Filtered_area_µm.tiff', np.rot90(np.flip(self.filter_area_metric, 0), k=1, axes=(1, 0))) # Area map in µm
            tf.imwrite(Metric_SubPathDir + '/Filtered_equivalent_diameter_µm.tiff', np.rot90(np.flip(self.filter_labeldiameter_metric, 0), k=1, axes=(1, 0)))

        # Saving cluster if asked
        if self.Save_cluster.isChecked(): # If QCheckBox 'Save clustered profiles' is True, then the function is run
            self.Compute_clustered_profiles() 
            tf.imwrite(PathDir + '/Clustered_profiles.tiff', self.liste)
            tf.imwrite(PathDir + '/Labeled_img_NoGB.tiff', np.rot90(np.flip(self.Labels_int, 0), k=1, axes=(1, 0)).astype('float32')) 

        # CSV save step
        self.extract_value_list()
        
        np.savetxt(Pxls_SubPathDir + "/Grain_area_list_pxls.csv", self.area_list_pxls, delimiter = ",")
        np.savetxt(Pxls_SubPathDir + "/Grain_size_list_pxls.csv", self.extract_diameter, delimiter = ",")
        
        np.savetxt(Pxls_SubPathDir + "/Filtered_grain_area_list_pxls.csv", self.Filtered_area_list_pxls, delimiter = ",")
        np.savetxt(Pxls_SubPathDir + "/Filtered_grain_size_list_pxls.csv", self.filtered_diameter, delimiter = ",")                           

        np.savetxt(Pxls_SubPathDir + "/Form_factor_list.csv", self.form_factor_list, delimiter = ",")
        np.savetxt(Pxls_SubPathDir + "/Filtered_Form_factor_list.csv", self.Filtered_form_factor_list, delimiter = ",")
        
        if self.flag_PixelSize == True:
            np.savetxt(Metric_SubPathDir + "/Grain_area_list_µm.csv", self.area_list_metric, delimiter = ",")
            np.savetxt(Metric_SubPathDir + "/Grain_size_list_µm.csv", self.extract_diameter_metric, delimiter = ",")
            
            np.savetxt(Metric_SubPathDir + "/Filtered_grain_area_list_µm.csv", self.Filtered_area_list_metric, delimiter = ",")
            np.savetxt(Metric_SubPathDir + "/Filtered_grain_size_list_µm.csv", self.filtered_diameter_metric, delimiter = ",")

        # Finished message
        self.parent.popup_message("Grain boundaries determination","Saving process is over.",'icons/Grain_Icons.png')

    def validate_data(self): # Push labeled image in the main GUI
        self.parent.Label_image = np.copy(self.Corrected_label_img) # Copy in the main GUI
        self.parent.StackList.append(self.Corrected_label_img) # Add the data in the stack list
        
        Combo_text = '\u2022 Grain labeling'
        Combo_data = self.Corrected_label_img
        self.parent.choiceBox.addItem(Combo_text, Combo_data) # Add the data in the QComboBox

        self.parent.displayDataview(self.parent.Label_image) # Display the labeled grain
        self.parent.choiceBox.setCurrentIndex(self.parent.choiceBox.count() - 1) # Show the last data in the choiceBox QComboBox

        self.parent.Info_box.ensureCursorVisible()
        self.parent.Info_box.insertPlainText("\n \u2022 Grain labeled.") 
        
        self.parent.flag_labeling = True
        
        self.parent.Save_button.setEnabled(True)
        self.parent.Reload_button.setEnabled(True)
        self.parent.choiceBox.setEnabled(True)
        
        # Finished message
        self.parent.popup_message("Grain boundaries determination","Labeled image has been exported to the main GUI.",'icons/Grain_Icons.png')

    def displayExpKAD(self, series): # Display of initial map
        self.KADSeries.addItem(self.crosshair_v1, ignoreBounds=True)
        self.KADSeries.addItem(self.crosshair_h1, ignoreBounds=True) 
        
        self.KADSeries.ui.histogram.hide()
        self.KADSeries.ui.roiBtn.hide()
        self.KADSeries.ui.menuBtn.hide()
        
        view = self.KADSeries.getView()
        state = view.getState()        
        self.KADSeries.setImage(series) 
        view.setState(state)
        view.setBackgroundColor(self.parent.color1)
        
        self.KADSeries.autoRange()
        
    def displayFilteredKAD(self, series): # Display of filtered map
        self.FiltKADSeries.addItem(self.crosshair_v2, ignoreBounds=True)
        self.FiltKADSeries.addItem(self.crosshair_h2, ignoreBounds=True) 
        
        self.FiltKADSeries.ui.histogram.hide()
        self.FiltKADSeries.ui.roiBtn.hide()
        self.FiltKADSeries.ui.menuBtn.hide()
        
        view = self.FiltKADSeries.getView()
        state = view.getState()        
        self.FiltKADSeries.setImage(series) 
        view.setState(state)
        view.setBackgroundColor(self.parent.color1)
        
    def displayOtsu1(self, series): # Display of Otsu1 map
        self.Otsu1Series.addItem(self.crosshair_v3, ignoreBounds=True)
        self.Otsu1Series.addItem(self.crosshair_h3, ignoreBounds=True) 
        
        self.Otsu1Series.ui.histogram.show()
        self.Otsu1Series.ui.roiBtn.hide()
        self.Otsu1Series.ui.menuBtn.hide()
        
        view = self.Otsu1Series.getView()
        state = view.getState()        
        self.Otsu1Series.setImage(series) 
        view.setState(state)
        view.setBackgroundColor(self.parent.color1)
        
        histplot = self.Otsu1Series.getHistogramWidget()
        histplot.setBackground(self.parent.color1)
        
        histplot.region.setBrush(pg.mkBrush(self.parent.color5 + (120,)))
        histplot.region.setHoverBrush(pg.mkBrush(self.parent.color5 + (60,)))
        histplot.region.pen = pg.mkPen(self.parent.color5)
        histplot.region.lines[0].setPen(pg.mkPen(self.parent.color5, width=2))
        histplot.region.lines[1].setPen(pg.mkPen(self.parent.color5, width=2))
        histplot.fillHistogram(color = self.parent.color5)        
        histplot.autoHistogramRange()
        
        self.Otsu1Series.setColorMap(pg.colormap.get('viridis'))
                
    def displayBinary1(self, series): # Display of binary1 map
        self.Binary1Series.addItem(self.crosshair_v4, ignoreBounds=True)
        self.Binary1Series.addItem(self.crosshair_h4, ignoreBounds=True) 
        
        self.Binary1Series.ui.histogram.hide()
        self.Binary1Series.ui.roiBtn.hide()
        self.Binary1Series.ui.menuBtn.hide()
        
        view = self.Binary1Series.getView()
        state = view.getState()        
        self.Binary1Series.setImage(series) 
        view.setState(state)
        view.setBackgroundColor(self.parent.color1)
        
    def displaylabels(self, series): # Display of label map
        self.LabelsSeries.addItem(self.crosshair_v5, ignoreBounds=True)
        self.LabelsSeries.addItem(self.crosshair_h5, ignoreBounds=True) 
        
        self.LabelsSeries.ui.histogram.show()
        self.LabelsSeries.ui.roiBtn.hide()
        self.LabelsSeries.ui.menuBtn.hide()
        
        view = self.LabelsSeries.getView()
        state = view.getState()        
        self.LabelsSeries.setImage(series) 
        view.setState(state)
        view.setBackgroundColor(self.parent.color1)
        
        histplot = self.LabelsSeries.getHistogramWidget()
        histplot.setBackground(self.parent.color1)
        
        histplot.region.setBrush(pg.mkBrush(self.parent.color5 + (120,)))
        histplot.region.setHoverBrush(pg.mkBrush(self.parent.color5 + (60,)))
        histplot.region.pen = pg.mkPen(self.parent.color5)
        histplot.region.lines[0].setPen(pg.mkPen(self.parent.color5, width=2))
        histplot.region.lines[1].setPen(pg.mkPen(self.parent.color5, width=2))
        histplot.fillHistogram(color = self.parent.color5)        
        histplot.autoHistogramRange()   
        
        self.LabelsSeries.setColorMap(pg.colormap.get('viridis'))

    def defaultIV(self):
        # KADSeries: Initial map
        self.KADSeries.ui.histogram.hide()
        self.KADSeries.ui.roiBtn.hide()
        self.KADSeries.ui.menuBtn.hide()
        
        view = self.KADSeries.getView()
        view.setBackgroundColor(self.parent.color1)
        
        # FiltKADSeries: map after filtering
        self.FiltKADSeries.ui.histogram.hide()
        self.FiltKADSeries.ui.roiBtn.hide()
        self.FiltKADSeries.ui.menuBtn.hide()
        
        view = self.FiltKADSeries.getView()
        view.setBackgroundColor(self.parent.color1)
        
        # Otsu1Series: Otsu n°1 classes definition
        self.Otsu1Series.ui.histogram.hide()
        self.Otsu1Series.ui.roiBtn.hide()
        self.Otsu1Series.ui.menuBtn.hide()
        
        view = self.Otsu1Series.getView()
        view.setBackgroundColor(self.parent.color1)
        
        # Binary1Series: Otsu n°1 after binarisation
        self.Binary1Series.ui.histogram.hide()
        self.Binary1Series.ui.roiBtn.hide()
        self.Binary1Series.ui.menuBtn.hide()
        
        view = self.Binary1Series.getView()
        view.setBackgroundColor(self.parent.color1)
        
        # LabelsSeries: grain labeling
        self.LabelsSeries.ui.histogram.hide()
        self.LabelsSeries.ui.roiBtn.hide()
        self.LabelsSeries.ui.menuBtn.hide()
        
        view = self.LabelsSeries.getView()
        view.setBackgroundColor(self.parent.color1)
        
    def mouseMoved(self, e):
        pos = e[0]
        sender = self.sender()
  
        if not self.mouseLock.isChecked():
            if self.KADSeries.view.sceneBoundingRect().contains(pos)\
                or self.FiltKADSeries.view.sceneBoundingRect().contains(pos)\
                or self.Otsu1Series.view.sceneBoundingRect().contains(pos)\
                or self.Binary1Series.view.sceneBoundingRect().contains(pos)\
                or self.LabelsSeries.view.sceneBoundingRect().contains(pos):    
                
                if sender == self.proxy1:
                    item = self.KADSeries.view
                elif sender == self.proxy3:
                    item = self.FiltKADSeries.view
                elif sender == self.proxy5:
                    item = self.Otsu1Series.view
                elif sender == self.proxy7:
                    item = self.Binary1Series.view
                else:
                    item = self.LabelsSeries.view
                
                mousePoint = item.mapSceneToView(pos) 
                     
                self.crosshair_v1.setPos(mousePoint.x())
                self.crosshair_h1.setPos(mousePoint.y())
                
                self.crosshair_v2.setPos(mousePoint.x())
                self.crosshair_h2.setPos(mousePoint.y())
                
                self.crosshair_v3.setPos(mousePoint.x())
                self.crosshair_h3.setPos(mousePoint.y())
                
                self.crosshair_v4.setPos(mousePoint.x())
                self.crosshair_h4.setPos(mousePoint.y())
                
                self.crosshair_v5.setPos(mousePoint.x())
                self.crosshair_h5.setPos(mousePoint.y())

            try:
                self.x = int(mousePoint.x())
                self.y = int(mousePoint.y())
                
                self.printClick(self.x, self.y, sender)
            except:
                pass
    
    def mouseClick(self, e):
        pos = e[0]
        
        self.mouseLock.toggle()
        
        fromPosX = pos.scenePos()[0]
        fromPosY = pos.scenePos()[1]
        
        posQpoint = QtCore.QPointF()
        posQpoint.setX(fromPosX)
        posQpoint.setY(fromPosY)

        if self.KADSeries.view.sceneBoundingRect().contains(posQpoint):
                
            item = self.KADSeries.view
            mousePoint = item.mapSceneToView(posQpoint) 

            self.crosshair_v1.setPos(mousePoint.x())
            self.crosshair_h1.setPos(mousePoint.y())
                 
            self.x = int(mousePoint.x())
            self.y = int(mousePoint.y())
            
    def printClick(self, x, y, sender):
        if self.flag_info == True:
            try:
                self.Otsu1_label.setText("Otsu classes: " + str(self.regions[x, y]))
            except:
                pass
            
        if self.flag_info_labels == False:
            try:
                self.GrainQLabel.setText("Label n°: " + str(self.Corrected_label_img[x, y]))        
            except:
                pass
        
        if self.flag_info_labels == True:
            try:
                self.GrainQLabel.setText("Equivalent \u2300 (pxls): " + str(np.round(self.Corrected_img_diameter[x, y],2)))        
            except:
                pass
            
        if self.flag_info_labels_metric == True:
            try:
                self.GrainQLabel.setText("Equivalent \u2300 (µm): " + str(np.round(self.Corrected_img_diameter_metric[x, y],2)))        
            except:
                pass
            
        if self.flag_info_overlay == True:
            try:
                self.GrainQLabel.setText("Map: " + str(np.round(self.overlay_KAD_GB[x, y],2)))        
            except:
                pass
            
        if self.flag_info_filtered == True:
            try:
                self.GrainQLabel.setText("Equivalent \u2300 (pxls): " + str(np.round(self.filter_labeldiameter[x, y],2)))        
            except:
                pass
            
        if self.flag_info_filtered2 == True:
            try:
                self.GrainQLabel.setText("Equivalent \u2300 (µm): " + str(np.round(self.filter_labeldiameter_metric[x, y],2)))        
            except:
                pass